import * as L from "leaflet";
import * as G from "geojson";
import LajiMap from "./map";
import { DrawOptions, DataItemType, LajiMapEvent, TileLayerOptions } from "./map.defs";
import { ControlOptions, ControlsOptions, CustomControl, InternalControlsOptions } from "./controls.defs";
declare type Constructor<LM> = new (...args: any[]) => LM;
export default function LajiMapWithControls<LM extends Constructor<LajiMap>>(Base: LM): {
    new (...args: any[]): {
        controls: L.Control[];
        _customControls: CustomControl[];
        layerControl: L.Control.Layers;
        controlItems: ControlOptions[];
        _controlItemsByName: {
            [controlName: string]: ControlOptions;
        };
        activeControl: L.Control;
        controlSettings: InternalControlsOptions;
        drawControl: L.Control.Draw;
        _locateOn: boolean;
        _controlButtons: {
            [controlName: string]: HTMLElement;
        };
        _opacitySetBySlide: boolean;
        _slider: any;
        _contextMenuItems: {
            [buttonName: string]: HTMLElement;
        };
        _internalTileLayersUpdate: boolean;
        getOptionKeys(): any;
        _setLang(): void;
        setLang(lang: any): void;
        _initializeMapEvents(): void;
        _toggleLocate(): void;
        _setLocateOn(...params: any[]): void;
        _setLocateOff(): void;
        _onLocationFound(e: any): void;
        setTileLayers(options: any): void;
        _updateUserLocate(value: any): void;
        _updateMapControls(): void;
        _updateDrawControls(): void;
        updateDrawData(item: DrawOptions): void;
        _updateCustomControls(): void;
        _updateLineTransectControls(): void;
        setControlsWarn(controlSettings: any): void;
        setControls(controlSettings: ControlsOptions | boolean): void;
        setCustomControls(controls?: CustomControl[]): void;
        _controlIsAllowed(name: any, custom?: boolean): any;
        _addControl(name: any, control: L.Control): void;
        _createControlButton(that: any, container: any, fn: any, name?: any): HTMLElement;
        _createControlItem(that: any, container: any, glyphName: any, title: any, fn: any, name: any): HTMLElement;
        _getDrawControl(): L.Control.Draw;
        _getCoordinatesControl(): L.Control;
        _getLayerControl(): L.Control;
        getZoomControl(): L.Control.Zoom;
        setTileLayerOpacity(opacity: any, triggerEvent?: any): void;
        setLineTransectGeometry(feature: any, undo: any): void;
        resetDrawUndoStack(): void;
        _updateDrawUndoStack(events: LajiMapEvent[] | LajiMapEvent, prevFeatureCollection: any, prevActiveIdx?: any): void;
        drawUndo(): void;
        drawRedo(): void;
        _updateUndoButton(buttonName: any, history: any, historyPointer: any): void;
        _updateRedoButton(buttonName: any, history: any, historyPointer: any): void;
        updateLTUndoButton(): void;
        updateLTRedoButton(): void;
        updateDrawUndoButton(): void;
        updateDrawRedoButton(): void;
        openCoordinatesInputDialog(): void;
        openDrawCopyDialog(): void;
        createFormatDetectorElem(options?: {
            displayFormat?: boolean;
            displayErrors?: boolean;
            allowGrid?: boolean;
        }): {
            elem: HTMLElement;
            validate: (value?: string) => {
                valid: boolean;
                geoJSON: G.GeoJSON;
            };
            unmount: () => void;
        };
        openDrawUploadDialog(): void;
        _joinTranslations(...words: any[]): string;
        _updateContextMenu(): void;
        getGoogleGeocodingControl(): any;
        setFullscreenOn(): void;
        setFullscreenOff(): void;
        _updateFullscreenControl(): void;
        triggerDrawing(featureType: DataItemType): void;
        shouldNotPreventScrolling(): boolean;
        googleApiKey: string;
        container: HTMLElement;
        mapElem: HTMLElement;
        blockerElem: HTMLElement;
        rootElem: HTMLElement;
        translations: any;
        data: import("./map.defs").Data[];
        map: L.Map;
        _onDrawReverse: (layer: import("./map.defs").DataItemLayer) => void;
        _onDrawRemove: (layer: import("./map.defs").DataItemLayer) => void;
        idxsToIds: {
            [dataIdx: number]: {
                [featureIdx: number]: number;
            };
        };
        idsToIdxs: {
            [dataIdx: number]: {
                [id: number]: number;
            };
        };
        idsToIdxTuples: {
            [id: number]: import("./map.defs").IdxTuple;
        };
        editIdxTuple: import("./map.defs").IdxTuple;
        drawing: boolean;
        _drawHistoryPointer: number;
        _drawHistory: import("./map.defs").DrawHistoryEntry[];
        polyline: L.Polyline<G.LineString | G.MultiLineString, any>;
        userLocation: {
            latlng: L.LatLng;
            accuracy: number;
        };
        userLocationMarker: L.CircleMarker<any>;
        userLocationRadiusMarker: L.Circle<any>;
        userLocationLayer: L.LayerGroup<any>;
        popup: any;
        featurePopupOffset: number;
        markerPopupOffset: number;
        lang: import("./map.defs").Lang;
        tileLayerOpacity: number;
        onSetLangHooks: (() => void)[];
        options: import("./map.defs").Options;
        _dialogRoot: HTMLElement;
        _openDialogs: any[];
        _closeDialog: (e?: Event) => void;
        draw: import("./map.defs").Draw;
        drawIdx: number;
        _draftDrawLayer: import("./map.defs").DataItemLayer;
        _idxsToHovered: {
            [dataIdx: number]: {
                [featureIdx: number]: boolean;
            };
        };
        _idxsToContextMenuOpen: {
            [dataIdx: number]: {
                [id: number]: boolean;
            };
        };
        locate: boolean | [(event: L.LocationEvent) => void, (error: L.ErrorEvent) => void];
        _located: boolean;
        popupOnHover: boolean;
        popupCounter: number;
        onPopupClose: () => void;
        _drawReverseLayers: L.Polyline<G.LineString | G.MultiLineString, any>[];
        _drawRemoveLayers: import("./map.defs").DataItemLayer[];
        _zoomToData: boolean | import("./map.defs").LajiMapFitBoundsOptions;
        _disableDblClickZoom: boolean;
        dictionary: {
            [lang: string]: any;
        };
        _domCleaners: (() => void)[];
        _documentEvents: {
            [eventName: string]: EventListener;
        };
        zoom: number;
        center: L.LatLngExpression;
        tileLayer: L.TileLayer;
        overlaysByNames: {
            [name: string]: L.TileLayer;
        };
        availableOverlaysByNames: {
            [name: string]: L.TileLayer;
        };
        overlays: L.TileLayer[];
        finnishTileLayers: {
            ykjGrid?: L.TileLayer;
            ykjGridLabels?: L.TileLayer;
            maastokartta?: L.TileLayer;
            taustakartta?: L.TileLayer;
            pohjakartta?: L.TileLayer;
            ortokuva?: L.TileLayer;
            laser?: L.TileLayer;
        };
        worldTileLayers: {
            openStreetMap?: L.TileLayer;
            googleSatellite?: L.TileLayer;
        };
        tileLayers: {
            ykjGrid?: L.TileLayer;
            ykjGridLabels?: L.TileLayer;
            maastokartta?: L.TileLayer;
            taustakartta?: L.TileLayer;
            pohjakartta?: L.TileLayer;
            ortokuva?: L.TileLayer;
            laser?: L.TileLayer;
            openStreetMap?: L.TileLayer;
            googleSatellite?: L.TileLayer;
            geobiologicalProvinces?: L.TileLayer;
            geobiologicalProvinceBorders?: L.TileLayer;
            municipalities?: L.TileLayer;
            forestVegetationZones?: L.TileLayer;
            mireVegetationZones?: L.TileLayer;
            threatenedSpeciesEvaluationZones?: L.TileLayer;
            biodiversityForestZones?: L.TileLayer;
        };
        tileLayerName: import("./map.defs").TileLayerName;
        _tileLayers: import("./map.defs").InternalTileLayersOptions;
        availableTileLayers: {
            [name: string]: L.TileLayer;
        };
        _listenedEvents: L.LeafletEventHandlerFnMap;
        _keyListeners: {
            [eventName: string]: {
                [key: string]: ((e: Event) => boolean | void)[];
            };
        };
        _swapToWorldFlag: boolean;
        _mouseLatLng: L.LatLng;
        _contextMenuLayer: L.Layer;
        _preventScrollDomCleaner: () => void;
        _onDrawStopPreventScrolling: () => void;
        _onDrawStartPreventScrolling: () => void;
        _preventScroll: boolean;
        _scrollPreventElem: HTMLElement;
        _scrollPreventTextElemContainer: HTMLElement;
        _scrollPreventTextElem: HTMLElement;
        _scrollPreventScrollListeners: [string, EventListener][];
        _startPreventScrollingTimeout: () => void;
        _onControlClickPreventScrolling: () => boolean;
        _onMouseDownPreventScrolling: EventListener;
        _onTouchPreventScrolling: EventListener;
        _preventScrolling: () => void;
        _showPreventAnimationTimeout: any;
        _showPreventShowTimeout: any;
        _showPreventHideTimeout: any;
        _showingPreventScroll: boolean;
        _LTEditPointIdxTuple: import("./map.defs").IdxTuple;
        _clickBeforeZoomAndPan: boolean;
        _origLatLngs: {
            [id: string]: L.LatLng[];
        };
        _tooltip: L.Draw.Tooltip;
        _tooltipTranslationHook: () => void;
        _onMouseMove: (e: any) => void;
        provider: any;
        leafletOptions: L.MapOptions;
        _viewCriticalSection: number;
        _tileLayersSet: boolean;
        activeProjName: "finnish" | "world";
        _tileLayerOrder: string[];
        viewLocked: boolean;
        _locateParam: boolean | import("./map.defs").UserLocationOptions;
        locateOptions: import("./map.defs").UserLocationOptions;
        availableTileLayersWhitelist: import("./map.defs").TileLayerName[];
        availableTileLayersBlacklist: import("./map.defs").TileLayerName[];
        _initialized: boolean;
        mmlProj: L.Proj.CRS;
        _trySwapToFinnishOnInitialization: boolean;
        _fullscreen: boolean;
        _fullscreenElem: HTMLElement;
        _beforeFullscreen: {
            rootElem: HTMLElement;
            bodyAsDialogRoot: boolean;
            clickBeforeZoomAndPan: boolean;
            bodyOverflowY: string;
        };
        setOptions(options?: import("./map.defs").Options): void;
        setOption(option: "center" | "data" | "circle" | "marker" | "polygon" | "polyline" | "contextmenu" | "zoomToData" | "locate" | "rootElem" | "lang" | "draw" | "tileLayerName" | "tileLayers" | "availableTileLayerNamesBlacklist" | "availableTileLayerNamesWhitelist" | "overlayNames" | "availableOverlayNameBlacklist" | "availableOverlayNameWhitelist" | "tileLayerOpacity" | "zoom" | "onPopupClose" | "markerPopupOffset" | "featurePopupOffset" | "popupOnHover" | "on" | "rectangle" | "bodyAsDialogRoot" | "clickBeforeZoomAndPan" | "viewLocked" | "controls" | "preferCanvas" | "attributionControl" | "zoomControl" | "closePopupOnClick" | "zoomSnap" | "zoomDelta" | "trackResize" | "boxZoom" | "doubleClickZoom" | "dragging" | "crs" | "minZoom" | "maxZoom" | "layers" | "maxBounds" | "renderer" | "fadeAnimation" | "markerZoomAnimation" | "transform3DLimit" | "zoomAnimation" | "zoomAnimationThreshold" | "inertia" | "inertiaDeceleration" | "inertiaMaxSpeed" | "easeLinearity" | "worldCopyJump" | "maxBoundsViscosity" | "keyboard" | "keyboardPanDelta" | "scrollWheelZoom" | "wheelDebounceTime" | "wheelPxPerZoomLevel" | "tap" | "tapTolerance" | "touchZoom" | "bounceAtZoomLimits" | "drawControl" | "smoothWheelZoom" | "smoothSensitivity" | "contextmenuInheritItems" | "contextmenuItems" | "contextmenuWidth", value: any): void;
        getOption(option: "center" | "data" | "circle" | "marker" | "polygon" | "polyline" | "contextmenu" | "zoomToData" | "locate" | "rootElem" | "lang" | "draw" | "tileLayerName" | "tileLayers" | "availableTileLayerNamesBlacklist" | "availableTileLayerNamesWhitelist" | "overlayNames" | "availableOverlayNameBlacklist" | "availableOverlayNameWhitelist" | "tileLayerOpacity" | "zoom" | "onPopupClose" | "markerPopupOffset" | "featurePopupOffset" | "popupOnHover" | "on" | "rectangle" | "bodyAsDialogRoot" | "clickBeforeZoomAndPan" | "viewLocked" | "controls" | "preferCanvas" | "attributionControl" | "zoomControl" | "closePopupOnClick" | "zoomSnap" | "zoomDelta" | "trackResize" | "boxZoom" | "doubleClickZoom" | "dragging" | "crs" | "minZoom" | "maxZoom" | "layers" | "maxBounds" | "renderer" | "fadeAnimation" | "markerZoomAnimation" | "transform3DLimit" | "zoomAnimation" | "zoomAnimationThreshold" | "inertia" | "inertiaDeceleration" | "inertiaMaxSpeed" | "easeLinearity" | "worldCopyJump" | "maxBoundsViscosity" | "keyboard" | "keyboardPanDelta" | "scrollWheelZoom" | "wheelDebounceTime" | "wheelPxPerZoomLevel" | "tap" | "tapTolerance" | "touchZoom" | "bounceAtZoomLimits" | "drawControl" | "smoothWheelZoom" | "smoothSensitivity" | "contextmenuInheritItems" | "contextmenuItems" | "contextmenuWidth"): any;
        getOptions(): import("./map.defs").Options;
        setRootElem(rootElem: HTMLElement): void;
        setBodyAsDialogRoot(value?: boolean): void;
        setClickBeforeZoomAndPan(value?: boolean): void;
        getMMLProj(): L.CRS;
        _initializeMap(): void;
        _initializeView(): void;
        _isOutsideFinland(latLng: L.LatLngExpression): boolean;
        _swapToWorldOutsideFinland(latLng: L.LatLngExpression): void;
        keyHandlerForType(type: string, e: any): boolean;
        _addDocumentEventListener(type: string, fn: EventListener): void;
        _addKeyListener(key: string, fn: (e?: Event) => boolean | void, type?: string): void;
        _removeKeyListener(key: string, fn: (e?: Event) => boolean | void, type?: string): void;
        _triggerKeyEvent(key: string, e: Event, type: string): boolean;
        setEventListeners(eventListeners: L.LeafletEventHandlerFnMap): void;
        _getDefaultCRSLayers(): L.TileLayer[];
        _getMMLCRSLayers(): L.TileLayer[];
        setTileLayerByName(name: import("./map.defs").TileLayerName): void;
        _getListForAvailableTileLayer(names: import("./map.defs").TileLayerName[], condition: any, _tileLayers?: {
            [name: string]: L.TileLayer;
        }): {};
        setAvailableTileLayers(names: import("./map.defs").TileLayerName[], condition: boolean, _tileLayers?: {
            [name: string]: L.TileLayer;
        }): void;
        getDefaultTileLayerBlacklist(): import("./map.defs").TileLayerName[];
        getDefaultTileLayerWhitelist(): import("./map.defs").TileLayerName[];
        setAvailableTileLayerBlacklist(names: import("./map.defs").TileLayerName[]): void;
        setAvailableTileLayerWhitelist(names: import("./map.defs").TileLayerName[]): void;
        getAvailableOverlaysByNames(): {
            [name: string]: L.TileLayer[];
        };
        getAvailableWorldTileLayers(): {
            [name: string]: L.TileLayer[];
        };
        getAvailableFinnishTileLayers(): {
            [name: string]: L.TileLayer[];
        };
        getAvailableLayersFor(_layers: any, availables: any): {
            [name: string]: L.TileLayer[];
        };
        setTileLayer(layer: L.TileLayer): void;
        getActiveLayers(options: import("./map.defs").InternalTileLayersOptions): {
            ykjGrid?: TileLayerOptions;
            ykjGridLabels?: TileLayerOptions;
            maastokartta?: TileLayerOptions;
            taustakartta?: TileLayerOptions;
            pohjakartta?: TileLayerOptions;
            ortokuva?: TileLayerOptions;
            laser?: TileLayerOptions;
            openStreetMap?: TileLayerOptions;
            googleSatellite?: TileLayerOptions;
            geobiologicalProvinces?: TileLayerOptions;
            geobiologicalProvinceBorders?: TileLayerOptions;
            municipalities?: TileLayerOptions;
            forestVegetationZones?: TileLayerOptions;
            mireVegetationZones?: TileLayerOptions;
            threatenedSpeciesEvaluationZones?: TileLayerOptions;
            biodiversityForestZones?: TileLayerOptions;
        };
        _getInternalTileLayerOptions(options: import("./map.defs").TileLayersOptions): import("./map.defs").InternalTileLayersOptions;
        getTileLayers(): import("./map.defs").TileLayersOptions;
        setOverlays(overlays?: L.TileLayer[]): void;
        _setOverlaysByName(overlayNames: import("./map.defs").OverlayName[]): void;
        getOverlaysByName(): import("./map.defs").OverlayName[];
        setOverlaysByName(overlayNames?: import("./map.defs").OverlayName[]): void;
        setAvailableOverlaysBlacklist(overlayNames: import("./map.defs").OverlayName[]): void;
        setAvailableOverlaysWhitelist(overlayNames: import("./map.defs").OverlayName[]): void;
        setAvailableOverlays(overlayNames: import("./map.defs").OverlayName[], condition: any): void;
        getNormalizedZoom(zoom?: number, tileLayer?: L.TileLayer): number;
        getDenormalizedZoom(zoom?: number, tileLayer?: L.TileLayer): number;
        initZoom(zoom: any): void;
        setNormalizedZoom(zoom: any, options?: {
            animate: boolean;
        }): void;
        setCenter(center: L.LatLngExpression): void;
        destroy(): void;
        cleanDOM(): void;
        _addDomCleaner(fn: () => void): void;
        _removeDomCleaner(fn: () => void): void;
        _constructDictionary(): void;
        formatFeatureOut(feature: G.Feature<G.Geometry, {
            [name: string]: any;
        }>, layer?: import("./map.defs").DataItemLayer): G.Feature<G.Geometry, {
            [name: string]: any;
        }>;
        formatFeatureIn(feature: G.Feature<G.Geometry, {
            [name: string]: any;
        }>, idx: number): G.Feature<G.Geometry, {
            [name: string]: any;
        }>;
        cloneFeatures(features: G.Feature<G.Geometry, {
            [name: string]: any;
        }>[]): G.Feature<G.Geometry, {
            [name: string]: any;
        }>[];
        initializeDataItem(options: import("./map.defs").DataOptions, dataIdx?: number): void;
        _getAllData(): {
            group: L.FeatureGroup<any>;
        }[];
        zoomToData(options?: boolean | import("./map.defs").ZoomToDataOptions): void;
        getBoundsForZoomToDataOptions(options?: boolean | import("./map.defs").ZoomToDataOptions): L.LatLngBounds;
        getBoundsForDraw(): L.LatLngBounds;
        getBoundsForIdxs(...idxs: any[]): L.LatLngBounds;
        getBoundsForData(datas?: {
            group: L.FeatureGroup<any>;
        }[]): L.LatLngBounds;
        getBoundsForLayers(layers: L.Layer[]): L.LatLngBounds;
        _mapLayersToBoundableLayers(layers: L.Layer[]): L.Layer[];
        _initializeLayer(layer: import("./map.defs").DataItemLayer, idxTuple: import("./map.defs").IdxTuple): void;
        fitBounds(_bounds: L.LatLngBoundsExpression, options?: import("./map.defs").LajiMapFitBoundsOptions): void;
        setData(data: import("./map.defs").Data | import("./map.defs").Data[]): void;
        getData: () => import("./map.defs").Data[];
        addData(items: import("./map.defs").DataOptions[]): void;
        updateData(dataIdx: any, item: import("./map.defs").DataOptions): void;
        removeData(dataIdx: number): void;
        getDrawOptions(options: boolean | DrawOptions): DrawOptions;
        setDraw(options: DrawOptions): void;
        getDraw(): import("./map.defs").Draw;
        drawIsAllowed(): boolean;
        drawIsEditable(): boolean;
        wrapGeoJSONCoordinate([lng, lat]: G.Position): G.Position;
        _setOnChangeForItem(item: any, format?: import("./map.defs").OnChangeCoordinateSystem, crs?: string): void;
        clearItemData(item: import("./map.defs").Data, commit?: boolean): void;
        clearDrawData(): void;
        _startDrawRemove(): void;
        _stopDrawRemove(): boolean;
        _finishDrawRemove(): void;
        _cancelDrawRemove(): boolean;
        _onDrawReverseHandler({ layer }: L.GeoJSONEvent): void;
        _startDrawReverse(): void;
        _stopDrawReverse(): void;
        _finishDrawReverse(): void;
        _cancelDrawReverse(): void;
        _createIcon(options?: L.PathOptions): L.Icon<L.IconOptions>;
        _getClusterIcon(data: import("./map.defs").Data): (cluster: L.MarkerCluster) => L.DivIcon;
        setActive(layer: import("./map.defs").DataItemLayer): void;
        _resetIds(dataIdx: number): void;
        _setIdForLayer(layer: import("./map.defs").DataItemLayer, idxTuple: import("./map.defs").IdxTuple): void;
        recluster(): void;
        _reclusterDrawData(): void;
        _reclusterData(): void;
        getClusterOptionsFor(item: import("./map.defs").Data): {
            showCoverageOnHover?: boolean;
            zoomToBoundsOnClick?: boolean;
            spiderfyOnMaxZoom?: boolean;
            removeOutsideVisibleBounds?: boolean;
            animate?: boolean;
            animateAddingMarkers?: boolean;
            disableClusteringAtZoom?: number;
            maxClusterRadius?: number | ((zoom: number) => number);
            polygonOptions?: L.PolylineOptions;
            singleMarkerMode?: boolean;
            spiderLegPolylineOptions?: L.PolylineOptions;
            spiderfyDistanceMultiplier?: number;
            iconCreateFunction: ((cluster: L.MarkerCluster) => L.Icon<L.IconOptions> | L.DivIcon) | ((cluster: L.MarkerCluster) => L.DivIcon);
            chunkedLoading?: boolean;
            chunkDelay?: number;
            pane?: string;
            attribution?: string;
        };
        _reclusterDataItem(item: import("./map.defs").Data): void;
        reorderData(): void;
        redrawDrawData(): void;
        redrawDataItem(idx: number): void;
        redrawData(): void;
        redraw(): void;
        _initializePopup(layer: import("./map.defs").DataItemLayer): void;
        _initializeTooltip(layer: import("./map.defs").DataItemLayer): void;
        _updateContextMenuForLayer(layer: import("./map.defs").DataItemLayer): void;
        setLocate(locate?: boolean | import("./map.defs").UserLocationOptions): void;
        createUserLocationLayer(latlng: L.LatLng, accuracy: number): {
            layerGroup: L.LayerGroup<any>;
            markerLayer: L.CircleMarker<any>;
            radiusLayer: L.Circle<any>;
        };
        _onLocationNotFound(e: L.ErrorEvent): void;
        getDrawLayerByIdx(idx: number): import("./map.defs").DataItemLayer;
        _getLayerByIdxTuple: (idxTuple: import("./map.defs").IdxTuple) => import("./map.defs").DataItemLayer;
        getLayerByIdxTuple(idxTuple: import("./map.defs").IdxTuple): import("./map.defs").DataItemLayer;
        _getLayerById(id: number): import("./map.defs").DataItemLayer;
        _getIdxTupleByLayer(layer: any): import("./map.defs").IdxTuple;
        _getDrawLayerById(id: any): import("./map.defs").DataItemLayer;
        _triggerEvent(e: any, handler: any): void;
        _decoratePolyline(layer: L.Polyline<G.LineString | G.MultiLineString, any>): void;
        _reversePolyline(layer: any): void;
        _getStartCircleStyle(lineLayer: any): L.PathOptions;
        _onAdd(dataIdx: number, layer: import("./map.defs").DataItemLayer, coordinateVerbatim?: string): void;
        _onEdit(dataIdx: any, data: any): void;
        _onDelete(dataIdx: any, deleteIds: any): void;
        _removeLayerFromItem(item: import("./map.defs").Data, layer: import("./map.defs").DataItemLayer): void;
        _onActiveChange(idxTuple: import("./map.defs").IdxTuple): void;
        focusToLayerByIdxs(idxTuple: import("./map.defs").IdxTuple): void;
        focusToDrawLayer(idx: number): void;
        _setEditable(layer: import("./map.defs").DataItemLayer): void;
        _clearEditable(): void;
        _commitEdit(): void;
        _interceptClick(): boolean;
        setLayerStyle(layer: L.FeatureGroup<any> | L.Polygon<any> | L.Polyline<G.LineString | G.MultiLineString, any> | L.Marker<any> | L.Circle<any>, style: L.PathOptions): void;
        _featureToLayer(getFeatureStyle: (options: import("./map.defs").GetFeatureStyleOptions) => L.PathOptions, dataIdx?: number): (feature: any) => any;
        _getDefaultDrawDraftStyle(): L.PathOptions;
        _fillStyleWithGlobals<T extends L.PathOptions>(idxTuple: import("./map.defs").IdxTuple): T;
        _getStyleForType(idx: number | import("./map.defs").IdxTuple, overrideStyles?: L.PathOptions): L.PathOptions;
        _getStyleForLayer(layer: import("./map.defs").DataItemLayer, overrideStyles?: any): L.PathOptions;
        updateLayerStyle(layer: import("./map.defs").DataItemLayer): void;
        _getDefaultDrawStyle(): L.PathOptions;
        _getDefaultDrawClusterStyle(): L.PathOptions;
        _getDefaultDataStyle(item: import("./map.defs").Data): () => L.PathOptions;
        _getDefaultDataClusterStyle: (item: any) => () => {
            color: string;
            opacity: number;
        };
        _getDefaultDraftStyle(dataIdx: number): L.PathOptions;
        _updateDataLayerGroupStyle(idx: number): void;
        addTranslationHook(elemOrFn: HTMLElement | (() => void), translationKey?: string | (() => string), attr?: string): () => void;
        removeTranslationHook(hook: () => void): void;
        _getDrawOptionsForType<T_1 extends L.PathOptions>(featureType: DataItemType): T_1;
        abortDrawing(e?: Event): void;
        addDrawAbortListeners(): void;
        addFeatureToDraw(feature: G.Feature<G.Geometry, {
            [name: string]: any;
        }>): void;
        addFeatureToData(feature: G.Feature<G.Geometry, {
            [name: string]: any;
        }>, dataIdx: number): void;
        getFeatureTypes(): DataItemType[];
        _showError(e: Error): void;
        showClosableElement(elem: HTMLElement, onClose: (e: Event) => void, blocker?: boolean, getContainer?: () => HTMLElement): void;
        _createTooltip(translationKey: string, error?: boolean): L.Draw.Tooltip;
        _disposeTooltip(): void;
        _showDialog(container: HTMLElement, onClose?: (e: Event) => void): void;
        setGoogleApiKey(googleApiKey: string): void;
        setGeocodingProvider(): void;
        geocode(query: string, additional?: any, zoom?: number): void;
        setViewLocked(value: any): void;
        toggleFullscreen(): void;
    };
} & LM;
export {};
