"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isProvided = exports.provide = exports.dependsOn = exports.reflect = exports.depsProvided = void 0;
var providerToDependency = {};
var dependencyToProvider = {};
var reflected = {};
/**
 * These utils allow functions to delay their initial execution until their dependencies have executed (they are
 * provided, as to speak).
 */
function depsProvided(target, name, args) {
    initDepContextFor(target);
    var depsExecuted = target.depsExecuted, params = target.params;
    if (!depsExecuted[name] && !depIsProvided(target, name)) {
        params[name] = args;
        return false;
    }
    return true;
}
exports.depsProvided = depsProvided;
/**
 * A decorator. If a function is reflected, it will execute always when some of it's dependencies is reprovided.
 */
function reflect() {
    return function (target, property) {
        reflected[property] = true;
    };
}
exports.reflect = reflect;
/**
 * A decorator. If the dependencies are not provided on initial execution, the initial execution MUST be delayed until
 * all dependencies are provided. When all dependencies are provided, the initial execution is executed.
 *
 * NOTE: You MUST manually check in the function body if the dependencies are provided! Just put this as the first line
 * of the function body:
 *
 *   if (!depsProvided(this, "functionName", arguments)) return;
 *
 */
function dependsOn() {
    var deps = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        deps[_i] = arguments[_i];
    }
    return function (target, property) {
        if (dependencyToProvider[property])
            return; // Dependency tree constructed already.
        deps.forEach(function (dep) {
            providerToDependency[dep] = __spreadArrays((providerToDependency[dep] || []), [property]);
        });
        dependencyToProvider[property] = deps;
    };
}
exports.dependsOn = dependsOn;
/**
 * Provide a dependency. All delayed functions depending on this function are executed after this.
 */
function provide(target, prov) {
    function executeDependencies(_target, _prov) {
        var depsExecuted = _target.depsExecuted;
        (providerToDependency[_prov] || []).filter(function (dep) { return depIsProvided(_target, dep); }).forEach(function (dep) {
            if (!_target.params[dep] && !reflected[dep])
                return;
            _target[dep].apply(_target, (_target.params[dep] || []));
            delete _target.params[dep];
            depsExecuted[dep] = true;
        });
    }
    initDepContextFor(target);
    target.provided[prov] = true;
    executeDependencies(target, prov);
}
exports.provide = provide;
function isProvided(target, prov) {
    return target.provided[prov];
}
exports.isProvided = isProvided;
function initDepContextFor(target) {
    ["provided", "depsExecuted", "params", "reflected"].forEach(function (prop) {
        if (!target[prop]) {
            target[prop] = {};
        }
    });
}
function depIsProvided(target, dep) {
    var returnValue = false;
    var depsExecuted = target.depsExecuted, provided = target.provided;
    if (dependencyToProvider[dep].every(function (_prov) { return provided[_prov]; })) {
        if (depsExecuted[dep] && !reflected[dep])
            return;
        returnValue = true;
    }
    return returnValue;
}
