import * as L from "leaflet";
import * as G from "geojson";
import "leaflet-geometryutil";
import LajiMap from "./map";
import { LineTransectEvent, LineTransectFeature, LineTransectGeometry, LineTransectHistoryEntry, LineTransectIdx, LineTransectOptions, PointIdxTuple, SegmentIdxTuple, SegmentLayer, TooltipMessages } from "./line-transect.defs";
import { IdxTuple } from "./map.defs";
declare type Constructor<LM> = new (...args: any[]) => LM;
export default function LajiMapWithLineTransect<LM extends Constructor<LajiMap>>(Base: LM): {
    new (...props: any[]): {
        _hoveredIdxTuple: IdxTuple;
        LTFeature: LineTransectFeature;
        _LTEditPointIdxTuple: IdxTuple;
        _splitIdxTuple: IdxTuple;
        _splitPoint: L.LatLng;
        _lineLayers: L.Polyline<G.LineString>[][];
        _pointLayers: L.CircleMarker[][];
        _pointIdsToIdxTuples: {
            [id: number]: IdxTuple;
        };
        _corridorLayers: L.Polygon<G.Polygon>[][];
        _allSegments: L.Polyline<G.LineString>[];
        _allCorridors: L.Polygon<G.Polygon>[];
        _allPoints: L.CircleMarker[];
        _lineSplitFn: (idxTuple: IdxTuple, splitPoint: L.LatLng) => void;
        _selectLTMode: "segment" | "line";
        _LTActiveIdx: number;
        _onLTChange: LineTransectOptions["onChange"];
        _LTDragging: boolean;
        _getLTFeatureStyle: LineTransectOptions["getFeatureStyle"];
        _getLTTooltip: LineTransectOptions["getTooltip"];
        _LTHistory: LineTransectHistoryEntry[];
        _LTHistoryPointer: number;
        _LTPrintMode: boolean;
        _LTEditable: boolean;
        _pointDragSnapMode: boolean;
        _dragCorridor: L.Polygon<G.Polygon>;
        _origLineTransect: L.FeatureGroup;
        _pointLayerGroup: L.FeatureGroup;
        _lineLayerGroup: L.FeatureGroup;
        _corridorLayerGroup: L.FeatureGroup;
        _overlappingNonadjacentPointIdxTuples: {
            [idxTupleString: string]: IdxTuple;
        };
        _overlappingAdjacentPointIdxTuples: {
            [idxTupleString: string]: IdxTuple;
        };
        _lineIdxsTupleStringsToLineGroupIdxs: {
            [idxTupleString: string]: number;
        };
        _groupIdxsToLineIdxs: {
            [groupIdx: number]: number[];
        };
        _LTStartText: L.Polyline;
        _LTGroups: L.FeatureGroup[];
        _tooltipIdx: number;
        _overlappingPointDialogSegmentIdxTuple: IdxTuple;
        leafletIdsToCorridorLineIdxs: {
            [id: string]: number;
        };
        leafletIdsToCorridorSegmentIdxs: {
            [id: string]: number;
        };
        leafletIdsToFlatCorridorSegmentIdxs: {
            [id: string]: number;
        };
        leafletIdsToFlatPointIdxs: {
            [id: string]: number;
        };
        lineIdsToCorridorIds: {
            [id: string]: number;
        };
        corridorFlatIdxsToLeafletIds: {
            [id: string]: number;
        };
        _hoveredType: "point" | "corridor";
        _LTClickTimeout: number;
        _closebyPointIdxTuple: IdxTuple;
        _pointLTShiftMode: boolean;
        _onSelectLT: (idxTuple: IdxTuple) => void;
        _firstLTSegmentToRemoveIdx: IdxTuple;
        _LTPointExpander: L.CircleMarker;
        _LTdragPoint: L.CircleMarker;
        _LTContextMenuLayer: SegmentLayer;
        _featureBeforePointDrag: LineTransectFeature;
        _LTPointLatLngBeforeDrag: L.LatLng;
        _hoveringDragPoint: boolean;
        _dragPointStart: L.LatLng;
        _dragMouseStart: L.LatLng;
        _cutLine: L.Polygon;
        _lineCutIdx: IdxTuple;
        _ltTooltip: L.Draw.Tooltip;
        messages: TooltipMessages;
        getOptionKeys(): any;
        _interceptClick(): boolean;
        _getAllData(): {
            group: L.FeatureGroup<any>;
        }[];
        setLineTransect(data: LineTransectOptions): void;
        setLTActiveIdx(idx: number): void;
        _formatLTFeatureOut(): LineTransectFeature;
        setLineTransectGeometry(geometry: LineTransectGeometry, events?: LineTransectEvent[]): void;
        _getLTStartTextCoordinates(segment: L.LatLng[]): L.LatLng[];
        LTUndo(): void;
        LTRedo(): void;
        _getTooltipForPointIdxTuple: (type: any, idxTuple: IdxTuple) => string;
        _clearTooltipDescription(): void;
        flatIdxToIdxTuple(idx: number): number[];
        _getPoint(idxTuple: IdxTuple, callback: (idxTuple: any) => void, questionTranslationKey?: string, firstTranslationKey?: string, lastTranslationKey?: string): void;
        getIdxsFromLayer(layer: SegmentLayer): LineTransectIdx;
        getIdxsFromEvent({ layer }: L.LayerEvent): LineTransectIdx;
        _setIdxTupleMappings(): void;
        _setLineTransectEvents(): void;
        _updateLTLayerContextMenus(): void;
        _getContextMenuForPoint(idxTuple: IdxTuple): L.Contextmenu.Options;
        removeLTPoint(idxTuple: IdxTuple, commit?: boolean | LineTransectEvent[]): any[];
        _setLTPointEditable(idxTuple: IdxTuple): void;
        _commitPointDrag(): void;
        _setPointDragStepMode(): void;
        _unsetPointDragStepMode(): void;
        _startLTDragHandler(handler: any): void;
        _stopLTDragHandler(handler: any): void;
        _startLTDragPointHandler({ latlng }: L.LeafletMouseEvent): void;
        _stopLTDragPointHandler(): void;
        _dragLTPointHandler({ latlng }: L.LeafletMouseEvent): void;
        _startLTDragCorridorHandler(e: L.LeafletMouseEvent): void;
        _stopLTDragCorridorHandler(): void;
        _dragLTCorridorHandler({ latlng }: L.LeafletMouseEvent): void;
        _dragLTHandler(latlng: L.LatLng): void;
        _getLTLayerForIdxTuple<T extends SegmentLayer>(layers: T[][], idxTuple: IdxTuple): T;
        _layerExistsForIdxTuple<T_2 extends SegmentLayer>(layer: T_2[][], idxTuple: IdxTuple): T_2;
        _getIdxTuplePrecedingPoint(idxTuple: IdxTuple): IdxTuple;
        _getIdxTuplePrecedingEditPoint(): IdxTuple;
        _getIdxTupleForLineFollowingPoint(idxTuple: IdxTuple): IdxTuple;
        _getIdxTupleFollowingEditPoint(): IdxTuple;
        _getIdxTupleForFollowingPoint(idxTuple: IdxTuple): IdxTuple;
        _degreesFromNorth(lineCoords: L.LatLngExpression[]): number;
        _getCorridorCoordsForLine(lineCoords: L.LatLngExpression[]): [L.LatLng, L.LatLng, L.LatLng, L.LatLng];
        _getOnActiveSegmentChangeEvent(lineIdx: number): LineTransectEvent;
        _getStyleForLTIdxTupleAndType(idxTuple: IdxTuple, type: Function): L.PathOptions;
        _setStyleForLTLayer(layer: SegmentLayer): void;
        _updateLTStyleForLineIdx(lineIdx: number): void;
        _updateLTStyleForIdxTuple(idxTuple: IdxTuple): void;
        _idxTupleToFlatIdx(idxTuple: IdxTuple): number;
        _commitLTLineSplit(idxTuple: IdxTuple, splitPoint: L.LatLng): void;
        _commitLTPointAdd(idxTuple: IdxTuple, splitPoint: any): void;
        stopLTLineSplit(): void;
        _mouseMoveLTLineSplitHandler({ latlng }: any): void;
        startLTLineSplit(): void;
        startLTLineSplitForIdx(idxTuple: IdxTuple): void;
        startLTPointAdd(): void;
        startLTPointAddSplitForIdx(idxTuple: IdxTuple): void;
        _LTPointIdxTupleIsGroupFirstOrLast(idxTuple: IdxTuple): boolean;
        _LTIdxTuplesAreFromSameGroup(idxTuple: IdxTuple, idxTuple2: IdxTuple): boolean;
        _pointCanBeShiftedTo(idxTuple: IdxTuple): boolean;
        startLTPointShift(): void;
        stopLTPointShift(): void;
        commitLTPointShift(idxTuple: IdxTuple): void;
        startSelectLTSegmentMode(onSelect: any, tooltip: any, mode?: "segment" | "line"): void;
        stopSelectLTSegmentMode(idxTuple?: IdxTuple): void;
        startRemoveLTPointMode(): void;
        stopRemoveLTPointMode(): void;
        chooseFirstSegmentToConnect(idxTuple: IdxTuple): boolean;
        chooseLastSegmentToConnectAndCommit(idxTuple: IdxTuple): boolean;
        splitLTByMeters(): void;
        _updateLTTooltip(): void;
        __updateLTTooltip(messages: TooltipMessages): void;
        _setLTPrintLines(): void;
        googleApiKey: string;
        container: HTMLElement;
        mapElem: HTMLElement;
        blockerElem: HTMLElement;
        rootElem: HTMLElement;
        translations: any;
        data: import("./map.defs").Data[];
        map: L.Map;
        _onDrawReverse: (layer: import("./map.defs").DataItemLayer) => void;
        _onDrawRemove: (layer: import("./map.defs").DataItemLayer) => void;
        idxsToIds: {
            [dataIdx: number]: {
                [featureIdx: number]: number;
            };
        };
        idsToIdxs: {
            [dataIdx: number]: {
                [id: number]: number;
            };
        };
        idsToIdxTuples: {
            [id: number]: IdxTuple;
        };
        editIdxTuple: IdxTuple;
        drawing: boolean;
        _drawHistoryPointer: number;
        _drawHistory: import("./map.defs").DrawHistoryEntry[];
        polyline: L.Polyline<G.LineString | G.MultiLineString, any>;
        userLocation: {
            latlng: L.LatLng;
            accuracy: number;
        };
        userLocationMarker: L.CircleMarker<any>;
        userLocationRadiusMarker: L.Circle<any>;
        userLocationLayer: L.LayerGroup<any>;
        popup: any;
        featurePopupOffset: number;
        markerPopupOffset: number;
        lang: import("./map.defs").Lang;
        tileLayerOpacity: number;
        onSetLangHooks: (() => void)[];
        options: import("./map.defs").Options;
        _dialogRoot: HTMLElement;
        _openDialogs: any[];
        _closeDialog: (e?: Event) => void;
        draw: import("./map.defs").Draw;
        drawIdx: number;
        _draftDrawLayer: import("./map.defs").DataItemLayer;
        _idxsToHovered: {
            [dataIdx: number]: {
                [featureIdx: number]: boolean;
            };
        };
        _idxsToContextMenuOpen: {
            [dataIdx: number]: {
                [id: number]: boolean;
            };
        };
        locate: boolean | [(event: L.LocationEvent) => void, (error: L.ErrorEvent) => void];
        _located: boolean;
        popupOnHover: boolean;
        popupCounter: number;
        onPopupClose: () => void;
        _drawReverseLayers: L.Polyline<G.LineString | G.MultiLineString, any>[];
        _drawRemoveLayers: import("./map.defs").DataItemLayer[];
        _zoomToData: boolean | import("./map.defs").LajiMapFitBoundsOptions;
        _disableDblClickZoom: boolean;
        dictionary: {
            [lang: string]: any;
        };
        _domCleaners: (() => void)[];
        _documentEvents: {
            [eventName: string]: EventListener;
        };
        zoom: number;
        center: L.LatLngExpression;
        tileLayer: L.TileLayer;
        overlaysByNames: {
            [name: string]: L.TileLayer;
        };
        availableOverlaysByNames: {
            [name: string]: L.TileLayer;
        };
        overlays: L.TileLayer[];
        finnishTileLayers: {
            ykjGrid?: L.TileLayer;
            ykjGridLabels?: L.TileLayer;
            maastokartta?: L.TileLayer;
            taustakartta?: L.TileLayer;
            pohjakartta?: L.TileLayer;
            ortokuva?: L.TileLayer;
            laser?: L.TileLayer;
        };
        worldTileLayers: {
            openStreetMap?: L.TileLayer;
            googleSatellite?: L.TileLayer;
        };
        tileLayers: {
            ykjGrid?: L.TileLayer;
            ykjGridLabels?: L.TileLayer;
            maastokartta?: L.TileLayer;
            taustakartta?: L.TileLayer;
            pohjakartta?: L.TileLayer;
            ortokuva?: L.TileLayer;
            laser?: L.TileLayer;
            openStreetMap?: L.TileLayer;
            googleSatellite?: L.TileLayer;
            geobiologicalProvinces?: L.TileLayer;
            geobiologicalProvinceBorders?: L.TileLayer;
            municipalities?: L.TileLayer;
            forestVegetationZones?: L.TileLayer;
            mireVegetationZones?: L.TileLayer;
            threatenedSpeciesEvaluationZones?: L.TileLayer;
            biodiversityForestZones?: L.TileLayer;
        };
        tileLayerName: import("./map.defs").TileLayerName;
        _tileLayers: import("./map.defs").InternalTileLayersOptions;
        availableTileLayers: {
            [name: string]: L.TileLayer;
        };
        _listenedEvents: L.LeafletEventHandlerFnMap;
        _keyListeners: {
            [eventName: string]: {
                [key: string]: ((e: Event) => boolean | void)[];
            };
        };
        _swapToWorldFlag: boolean;
        _mouseLatLng: L.LatLng;
        _contextMenuLayer: L.Layer;
        _preventScrollDomCleaner: () => void;
        _onDrawStopPreventScrolling: () => void;
        _onDrawStartPreventScrolling: () => void;
        _preventScroll: boolean;
        _scrollPreventElem: HTMLElement;
        _scrollPreventTextElemContainer: HTMLElement;
        _scrollPreventTextElem: HTMLElement;
        _scrollPreventScrollListeners: [string, EventListener][];
        _startPreventScrollingTimeout: () => void;
        _onControlClickPreventScrolling: () => boolean;
        _onMouseDownPreventScrolling: EventListener;
        _onTouchPreventScrolling: EventListener;
        _preventScrolling: () => void;
        _showPreventAnimationTimeout: any;
        _showPreventShowTimeout: any;
        _showPreventHideTimeout: any;
        _showingPreventScroll: boolean;
        _clickBeforeZoomAndPan: boolean;
        _origLatLngs: {
            [id: string]: L.LatLng[];
        };
        _tooltip: L.Draw.Tooltip;
        _tooltipTranslationHook: () => void;
        _onMouseMove: (e: any) => void;
        provider: any;
        leafletOptions: L.MapOptions;
        _viewCriticalSection: number;
        _tileLayersSet: boolean;
        activeProjName: "finnish" | "world";
        _tileLayerOrder: string[];
        viewLocked: boolean;
        _locateParam: boolean | import("./map.defs").UserLocationOptions;
        locateOptions: import("./map.defs").UserLocationOptions;
        availableTileLayersWhitelist: import("./map.defs").TileLayerName[];
        availableTileLayersBlacklist: import("./map.defs").TileLayerName[];
        _initialized: boolean;
        mmlProj: L.Proj.CRS;
        _trySwapToFinnishOnInitialization: boolean;
        _fullscreen: boolean;
        _fullscreenElem: HTMLElement;
        _beforeFullscreen: {
            rootElem: HTMLElement;
            bodyAsDialogRoot: boolean;
            clickBeforeZoomAndPan: boolean;
            bodyOverflowY: string;
        };
        setOptions(options?: import("./map.defs").Options): void;
        setOption(option: "center" | "data" | "circle" | "marker" | "polygon" | "polyline" | "contextmenu" | "zoomToData" | "locate" | "rootElem" | "lang" | "draw" | "tileLayerName" | "tileLayers" | "availableTileLayerNamesBlacklist" | "availableTileLayerNamesWhitelist" | "overlayNames" | "availableOverlayNameBlacklist" | "availableOverlayNameWhitelist" | "tileLayerOpacity" | "zoom" | "onPopupClose" | "markerPopupOffset" | "featurePopupOffset" | "popupOnHover" | "on" | "rectangle" | "bodyAsDialogRoot" | "clickBeforeZoomAndPan" | "viewLocked" | "controls" | "preferCanvas" | "attributionControl" | "zoomControl" | "closePopupOnClick" | "zoomSnap" | "zoomDelta" | "trackResize" | "boxZoom" | "doubleClickZoom" | "dragging" | "crs" | "minZoom" | "maxZoom" | "layers" | "maxBounds" | "renderer" | "fadeAnimation" | "markerZoomAnimation" | "transform3DLimit" | "zoomAnimation" | "zoomAnimationThreshold" | "inertia" | "inertiaDeceleration" | "inertiaMaxSpeed" | "easeLinearity" | "worldCopyJump" | "maxBoundsViscosity" | "keyboard" | "keyboardPanDelta" | "scrollWheelZoom" | "wheelDebounceTime" | "wheelPxPerZoomLevel" | "tap" | "tapTolerance" | "touchZoom" | "bounceAtZoomLimits" | "drawControl" | "smoothWheelZoom" | "smoothSensitivity" | "contextmenuInheritItems" | "contextmenuItems" | "contextmenuWidth", value: any): void;
        getOption(option: "center" | "data" | "circle" | "marker" | "polygon" | "polyline" | "contextmenu" | "zoomToData" | "locate" | "rootElem" | "lang" | "draw" | "tileLayerName" | "tileLayers" | "availableTileLayerNamesBlacklist" | "availableTileLayerNamesWhitelist" | "overlayNames" | "availableOverlayNameBlacklist" | "availableOverlayNameWhitelist" | "tileLayerOpacity" | "zoom" | "onPopupClose" | "markerPopupOffset" | "featurePopupOffset" | "popupOnHover" | "on" | "rectangle" | "bodyAsDialogRoot" | "clickBeforeZoomAndPan" | "viewLocked" | "controls" | "preferCanvas" | "attributionControl" | "zoomControl" | "closePopupOnClick" | "zoomSnap" | "zoomDelta" | "trackResize" | "boxZoom" | "doubleClickZoom" | "dragging" | "crs" | "minZoom" | "maxZoom" | "layers" | "maxBounds" | "renderer" | "fadeAnimation" | "markerZoomAnimation" | "transform3DLimit" | "zoomAnimation" | "zoomAnimationThreshold" | "inertia" | "inertiaDeceleration" | "inertiaMaxSpeed" | "easeLinearity" | "worldCopyJump" | "maxBoundsViscosity" | "keyboard" | "keyboardPanDelta" | "scrollWheelZoom" | "wheelDebounceTime" | "wheelPxPerZoomLevel" | "tap" | "tapTolerance" | "touchZoom" | "bounceAtZoomLimits" | "drawControl" | "smoothWheelZoom" | "smoothSensitivity" | "contextmenuInheritItems" | "contextmenuItems" | "contextmenuWidth"): any;
        getOptions(): import("./map.defs").Options;
        setRootElem(rootElem: HTMLElement): void;
        setBodyAsDialogRoot(value?: boolean): void;
        shouldNotPreventScrolling(): boolean;
        setClickBeforeZoomAndPan(value?: boolean): void;
        getMMLProj(): L.CRS;
        _initializeMap(): void;
        _initializeView(): void;
        _isOutsideFinland(latLng: L.LatLngExpression): boolean;
        _swapToWorldOutsideFinland(latLng: L.LatLngExpression): void;
        _initializeMapEvents(): void;
        keyHandlerForType(type: string, e: any): boolean;
        _addDocumentEventListener(type: string, fn: EventListener): void;
        _addKeyListener(key: string, fn: (e?: Event) => boolean | void, type?: string): void;
        _removeKeyListener(key: string, fn: (e?: Event) => boolean | void, type?: string): void;
        _triggerKeyEvent(key: string, e: Event, type: string): boolean;
        setEventListeners(eventListeners: L.LeafletEventHandlerFnMap): void;
        _getDefaultCRSLayers(): L.TileLayer[];
        _getMMLCRSLayers(): L.TileLayer[];
        setTileLayerByName(name: import("./map.defs").TileLayerName): void;
        _getListForAvailableTileLayer(names: import("./map.defs").TileLayerName[], condition: any, _tileLayers?: {
            [name: string]: L.TileLayer;
        }): {};
        setAvailableTileLayers(names: import("./map.defs").TileLayerName[], condition: boolean, _tileLayers?: {
            [name: string]: L.TileLayer;
        }): void;
        getDefaultTileLayerBlacklist(): import("./map.defs").TileLayerName[];
        getDefaultTileLayerWhitelist(): import("./map.defs").TileLayerName[];
        setAvailableTileLayerBlacklist(names: import("./map.defs").TileLayerName[]): void;
        setAvailableTileLayerWhitelist(names: import("./map.defs").TileLayerName[]): void;
        getAvailableOverlaysByNames(): {
            [name: string]: L.TileLayer[];
        };
        getAvailableWorldTileLayers(): {
            [name: string]: L.TileLayer[];
        };
        getAvailableFinnishTileLayers(): {
            [name: string]: L.TileLayer[];
        };
        getAvailableLayersFor(_layers: any, availables: any): {
            [name: string]: L.TileLayer[];
        };
        setTileLayer(layer: L.TileLayer): void;
        getActiveLayers(options: import("./map.defs").InternalTileLayersOptions): {
            ykjGrid?: import("./map.defs").TileLayerOptions;
            ykjGridLabels?: import("./map.defs").TileLayerOptions;
            maastokartta?: import("./map.defs").TileLayerOptions;
            taustakartta?: import("./map.defs").TileLayerOptions;
            pohjakartta?: import("./map.defs").TileLayerOptions;
            ortokuva?: import("./map.defs").TileLayerOptions;
            laser?: import("./map.defs").TileLayerOptions;
            openStreetMap?: import("./map.defs").TileLayerOptions;
            googleSatellite?: import("./map.defs").TileLayerOptions;
            geobiologicalProvinces?: import("./map.defs").TileLayerOptions;
            geobiologicalProvinceBorders?: import("./map.defs").TileLayerOptions;
            municipalities?: import("./map.defs").TileLayerOptions;
            forestVegetationZones?: import("./map.defs").TileLayerOptions;
            mireVegetationZones?: import("./map.defs").TileLayerOptions;
            threatenedSpeciesEvaluationZones?: import("./map.defs").TileLayerOptions;
            biodiversityForestZones?: import("./map.defs").TileLayerOptions;
        };
        _getInternalTileLayerOptions(options: import("./map.defs").TileLayersOptions): import("./map.defs").InternalTileLayersOptions;
        setTileLayers(options: import("./map.defs").TileLayersOptions): void;
        getTileLayers(): import("./map.defs").TileLayersOptions;
        setTileLayerOpacity(val: any, triggerEvent?: boolean): void;
        setOverlays(overlays?: L.TileLayer[]): void;
        _setOverlaysByName(overlayNames: import("./map.defs").OverlayName[]): void;
        getOverlaysByName(): import("./map.defs").OverlayName[];
        setOverlaysByName(overlayNames?: import("./map.defs").OverlayName[]): void;
        setAvailableOverlaysBlacklist(overlayNames: import("./map.defs").OverlayName[]): void;
        setAvailableOverlaysWhitelist(overlayNames: import("./map.defs").OverlayName[]): void;
        setAvailableOverlays(overlayNames: import("./map.defs").OverlayName[], condition: any): void;
        getNormalizedZoom(zoom?: number, tileLayer?: L.TileLayer): number;
        getDenormalizedZoom(zoom?: number, tileLayer?: L.TileLayer): number;
        initZoom(zoom: any): void;
        setNormalizedZoom(zoom: any, options?: {
            animate: boolean;
        }): void;
        setCenter(center: L.LatLngExpression): void;
        destroy(): void;
        cleanDOM(): void;
        _addDomCleaner(fn: () => void): void;
        _removeDomCleaner(fn: () => void): void;
        _constructDictionary(): void;
        setLang(lang: import("./map.defs").Lang, shouldProvide?: boolean): void;
        formatFeatureOut(feature: G.Feature<G.Geometry, {
            [name: string]: any;
        }>, layer?: import("./map.defs").DataItemLayer): G.Feature<G.Geometry, {
            [name: string]: any;
        }>;
        formatFeatureIn(feature: G.Feature<G.Geometry, {
            [name: string]: any;
        }>, idx: number): G.Feature<G.Geometry, {
            [name: string]: any;
        }>;
        cloneFeatures(features: G.Feature<G.Geometry, {
            [name: string]: any;
        }>[]): G.Feature<G.Geometry, {
            [name: string]: any;
        }>[];
        initializeDataItem(options: import("./map.defs").DataOptions, dataIdx?: number): void;
        zoomToData(options?: boolean | import("./map.defs").ZoomToDataOptions): void;
        getBoundsForZoomToDataOptions(options?: boolean | import("./map.defs").ZoomToDataOptions): L.LatLngBounds;
        getBoundsForDraw(): L.LatLngBounds;
        getBoundsForIdxs(...idxs: any[]): L.LatLngBounds;
        getBoundsForData(datas?: {
            group: L.FeatureGroup<any>;
        }[]): L.LatLngBounds;
        getBoundsForLayers(layers: L.Layer[]): L.LatLngBounds;
        _mapLayersToBoundableLayers(layers: L.Layer[]): L.Layer[];
        _initializeLayer(layer: import("./map.defs").DataItemLayer, idxTuple: IdxTuple): void;
        fitBounds(_bounds: L.LatLngBoundsExpression, options?: import("./map.defs").LajiMapFitBoundsOptions): void;
        setData(data: import("./map.defs").Data | import("./map.defs").Data[]): void;
        getData: () => import("./map.defs").Data[];
        addData(items: import("./map.defs").DataOptions[]): void;
        updateData(dataIdx: any, item: import("./map.defs").DataOptions): void;
        updateDrawData(item: import("./map.defs").DrawOptions): void;
        removeData(dataIdx: number): void;
        getDrawOptions(options: boolean | import("./map.defs").DrawOptions): import("./map.defs").DrawOptions;
        setDraw(options: import("./map.defs").DrawOptions): void;
        getDraw(): import("./map.defs").Draw;
        drawIsAllowed(): boolean;
        drawIsEditable(): boolean;
        resetDrawUndoStack(): void;
        drawUndo(): void;
        drawRedo(): void;
        wrapGeoJSONCoordinate([lng, lat]: G.Position): G.Position;
        _setOnChangeForItem(item: any, format?: import("./map.defs").OnChangeCoordinateSystem, crs?: string): void;
        clearItemData(item: import("./map.defs").Data, commit?: boolean): void;
        clearDrawData(): void;
        _startDrawRemove(): void;
        _stopDrawRemove(): boolean;
        _finishDrawRemove(): void;
        _cancelDrawRemove(): boolean;
        _onDrawReverseHandler({ layer }: L.GeoJSONEvent): void;
        _startDrawReverse(): void;
        _stopDrawReverse(): void;
        _finishDrawReverse(): void;
        _cancelDrawReverse(): void;
        _createIcon(options?: L.PathOptions): L.Icon<L.IconOptions>;
        _getClusterIcon(data: import("./map.defs").Data): (cluster: L.MarkerCluster) => L.DivIcon;
        setActive(layer: import("./map.defs").DataItemLayer): void;
        _resetIds(dataIdx: number): void;
        _setIdForLayer(layer: import("./map.defs").DataItemLayer, idxTuple: IdxTuple): void;
        recluster(): void;
        _reclusterDrawData(): void;
        _reclusterData(): void;
        getClusterOptionsFor(item: import("./map.defs").Data): {
            showCoverageOnHover?: boolean;
            zoomToBoundsOnClick?: boolean;
            spiderfyOnMaxZoom?: boolean;
            removeOutsideVisibleBounds?: boolean;
            animate?: boolean;
            animateAddingMarkers?: boolean;
            disableClusteringAtZoom?: number;
            maxClusterRadius?: number | ((zoom: number) => number);
            polygonOptions?: L.PolylineOptions;
            singleMarkerMode?: boolean;
            spiderLegPolylineOptions?: L.PolylineOptions;
            spiderfyDistanceMultiplier?: number;
            iconCreateFunction: ((cluster: L.MarkerCluster) => L.Icon<L.IconOptions> | L.DivIcon) | ((cluster: L.MarkerCluster) => L.DivIcon);
            chunkedLoading?: boolean;
            chunkDelay?: number;
            pane?: string;
            attribution?: string;
        };
        _reclusterDataItem(item: import("./map.defs").Data): void;
        reorderData(): void;
        redrawDrawData(): void;
        redrawDataItem(idx: number): void;
        redrawData(): void;
        redraw(): void;
        _initializePopup(layer: import("./map.defs").DataItemLayer): void;
        _initializeTooltip(layer: import("./map.defs").DataItemLayer): void;
        _updateContextMenuForLayer(layer: import("./map.defs").DataItemLayer): void;
        setLocate(locate?: boolean | import("./map.defs").UserLocationOptions): void;
        _setLocateOn(triggerEvent?: boolean): void;
        _setLocateOff(): void;
        _onLocationFound({ latlng, accuracy, bounds }: L.LocationEvent): void;
        createUserLocationLayer(latlng: L.LatLng, accuracy: number): {
            layerGroup: L.LayerGroup<any>;
            markerLayer: L.CircleMarker<any>;
            radiusLayer: L.Circle<any>;
        };
        _onLocationNotFound(e: L.ErrorEvent): void;
        getDrawLayerByIdx(idx: number): import("./map.defs").DataItemLayer;
        _getLayerByIdxTuple: (idxTuple: IdxTuple) => import("./map.defs").DataItemLayer;
        getLayerByIdxTuple(idxTuple: IdxTuple): import("./map.defs").DataItemLayer;
        _getLayerById(id: number): import("./map.defs").DataItemLayer;
        _getIdxTupleByLayer(layer: any): IdxTuple;
        _getDrawLayerById(id: any): import("./map.defs").DataItemLayer;
        _triggerEvent(e: any, handler: any): void;
        _decoratePolyline(layer: L.Polyline<G.LineString | G.MultiLineString, any>): void;
        _reversePolyline(layer: any): void;
        _getStartCircleStyle(lineLayer: any): L.PathOptions;
        _updateDrawUndoStack(events: import("./map.defs").LajiMapCreateEvent | import("./map.defs").LajiMapEditEvent | import("./map.defs").LajiMapDeleteEvent | import("./map.defs").LajiMapInsertEvent | import("./map.defs").LajiMapActivateEvent | import("./map.defs").LajiMapEvent[], prevFeatureCollection: any, prevActiveIdx?: any): void;
        _onAdd(dataIdx: number, layer: import("./map.defs").DataItemLayer, coordinateVerbatim?: string): void;
        _onEdit(dataIdx: any, data: any): void;
        _onDelete(dataIdx: any, deleteIds: any): void;
        _removeLayerFromItem(item: import("./map.defs").Data, layer: import("./map.defs").DataItemLayer): void;
        _onActiveChange(idxTuple: IdxTuple): void;
        focusToLayerByIdxs(idxTuple: IdxTuple): void;
        focusToDrawLayer(idx: number): void;
        _setEditable(layer: import("./map.defs").DataItemLayer): void;
        _clearEditable(): void;
        _commitEdit(): void;
        setLayerStyle(layer: L.FeatureGroup<any> | L.Polygon<any> | L.Polyline<G.LineString | G.MultiLineString, any> | L.Marker<any> | L.Circle<any>, style: L.PathOptions): void;
        _featureToLayer(getFeatureStyle: (options: import("./map.defs").GetFeatureStyleOptions) => L.PathOptions, dataIdx?: number): (feature: any) => any;
        _getDefaultDrawDraftStyle(): L.PathOptions;
        _fillStyleWithGlobals<T_4 extends L.PathOptions>(idxTuple: IdxTuple): T_4;
        _getStyleForType(idx: number | IdxTuple, overrideStyles?: L.PathOptions): L.PathOptions;
        _getStyleForLayer(layer: import("./map.defs").DataItemLayer, overrideStyles?: any): L.PathOptions;
        updateLayerStyle(layer: import("./map.defs").DataItemLayer): void;
        _getDefaultDrawStyle(): L.PathOptions;
        _getDefaultDrawClusterStyle(): L.PathOptions;
        _getDefaultDataStyle(item: import("./map.defs").Data): () => L.PathOptions;
        _getDefaultDataClusterStyle: (item: any) => () => {
            color: string;
            opacity: number;
        };
        _getDefaultDraftStyle(dataIdx: number): L.PathOptions;
        _updateDataLayerGroupStyle(idx: number): void;
        addTranslationHook(elemOrFn: HTMLElement | (() => void), translationKey?: string | (() => string), attr?: string): () => void;
        removeTranslationHook(hook: () => void): void;
        _getDrawOptionsForType<T_5 extends L.PathOptions>(featureType: import("./map.defs").DataItemType): T_5;
        abortDrawing(e?: Event): void;
        addDrawAbortListeners(): void;
        triggerDrawing(featureType: import("./map.defs").DataItemType): void;
        addFeatureToDraw(feature: G.Feature<G.Geometry, {
            [name: string]: any;
        }>): void;
        addFeatureToData(feature: G.Feature<G.Geometry, {
            [name: string]: any;
        }>, dataIdx: number): void;
        getFeatureTypes(): import("./map.defs").DataItemType[];
        _showError(e: Error): void;
        showClosableElement(elem: HTMLElement, onClose: (e: Event) => void, blocker?: boolean, getContainer?: () => HTMLElement): void;
        _createTooltip(translationKey: string, error?: boolean): L.Draw.Tooltip;
        _disposeTooltip(): void;
        _showDialog(container: HTMLElement, onClose?: (e: Event) => void): void;
        setGoogleApiKey(googleApiKey: string): void;
        setGeocodingProvider(): void;
        geocode(query: string, additional?: any, zoom?: number): void;
        setViewLocked(value: any): void;
        toggleFullscreen(): void;
        setFullscreenOn(): void;
        setFullscreenOff(): void;
    };
} & LM;
export {};
