"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
var L = require("leaflet");
var dependency_utils_1 = require("./dependency-utils");
var utils_1 = require("./utils");
require("leaflet-geometryutil");
var globals_1 = require("./globals");
var map_1 = require("./map");
var POINT_DIST_TRESHOLD = 50;
var ODD_AMOUNT = 30;
var lineStyle = { color: globals_1.NORMAL_COLOR, weight: 2 };
var activeLineStyle = __assign(__assign({}, lineStyle), { color: globals_1.ACTIVE_COLOR });
var hoverLineStyle = __assign(__assign({}, lineStyle), { color: utils_1.combineColors(lineStyle.color, activeLineStyle.color) });
var editLineStyle = __assign(__assign({}, lineStyle), { color: "#f00" });
var defaultLineStyle = __assign(__assign({}, lineStyle), { weight: 1, fillColor: "#99b" });
var corridorStyle = __assign(__assign({}, lineStyle), { fillOpacity: 0.6, weight: 0, fillColor: lineStyle.color });
var oddCorridorStyle = __assign(__assign({}, corridorStyle), { weight: 2, fillColor: utils_1.combineColors(lineStyle.color, "#000000", ODD_AMOUNT) }); // tslint:disable-line
var activeCorridorStyle = __assign(__assign({}, corridorStyle), { fillColor: activeLineStyle.color });
var editCorridorStyle = __assign(__assign({}, corridorStyle), { fillColor: editLineStyle.color, fillOpacity: 0.5 });
var hoverCorridorStyle = __assign(__assign({}, corridorStyle), { fillColor: hoverLineStyle.color });
var pointStyle = { weight: 0, radius: 3, fillColor: "#154EAA", fillOpacity: 1 };
var oddPointStyle = __assign(__assign({}, pointStyle), { fillColor: utils_1.combineColors(pointStyle.fillColor, "#000000", ODD_AMOUNT) }); // tslint:disable-line
var activePointStyle = __assign(__assign({}, pointStyle), { fillColor: utils_1.combineColors(activeLineStyle.color, "#000000", 40) }); // tslint:disable-line
var editPointStyle = __assign(__assign({}, pointStyle), { fillColor: editLineStyle.color });
var hoverPointStyle = __assign(__assign({}, pointStyle), { fillColor: hoverLineStyle.color });
var editablePointStyle = __assign(__assign({}, pointStyle), { radius: 5, fillColor: "#f00", fillOpacity: 0.7 });
var overlappingPointStyle = __assign(__assign({}, pointStyle), { radius: 5, weight: 3, color: "#000" });
var firstOverlappingPointStyle = __assign(__assign({}, overlappingPointStyle), { fillColor: "#f00" });
var seamPointStyle = __assign(__assign({}, pointStyle), { radius: 7 });
var closebyEditPointStyle = __assign(__assign({}, editPointStyle), { radius: 9 });
var closebyPointStyle = __assign(__assign({}, pointStyle), { fillColor: editablePointStyle.fillColor, radius: 9, fillOpacity: editablePointStyle.fillOpacity }); // tslint:disable-line
var hintPointStyle = __assign(__assign({}, closebyPointStyle), { radius: 7 });
var LT_WIDTH_METERS = 25;
function flattenMatrix(m) {
    return m.reduce(function (flattened, array) { return __spreadArrays(flattened, array); }, []);
}
function idxTuplesEqual(i, j) {
    if (i === undefined || j === undefined) {
        return i === j;
    }
    return i.every(function (_, idx) { return i[idx] === j[idx]; });
}
function lineToGeoJSONLine(line) {
    var firstLatLng = line[0].getLatLngs()[0];
    return line.reduce(function (geoJSON, segment) {
        var latLng = segment.getLatLngs()[1];
        geoJSON.coordinates.push([latLng.lng, latLng.lat]);
        return geoJSON;
    }, {
        type: "LineString",
        coordinates: [[firstLatLng.lng, firstLatLng.lat]]
    });
}
function idxTupleToIdxTupleStr(idxTuple) {
    var i = idxTuple[0], j = idxTuple[1];
    return i !== undefined && j !== undefined ? i + "-" + j : undefined;
}
function LajiMapWithLineTransect(Base) {
    var LajiMapWithLineTransect = /** @class */ (function (_super) {
        __extends(LajiMapWithLineTransect, _super);
        function LajiMapWithLineTransect() {
            var props = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                props[_i] = arguments[_i];
            }
            var _this = _super.apply(this, props) || this;
            _this._pointDragSnapMode = false;
            _this._getTooltipForPointIdxTuple = function (type, idxTuple) {
                var lineIdx = idxTuple[0], pointIdx = idxTuple[1];
                var getTooltipForLineIdx = function (_lineIdx) {
                    var _a = utils_1.getLineTransectStartEndDistancesForIdx(_this._formatLTFeatureOut(), _lineIdx, 10), prevDistance = _a[0], distance = _a[1];
                    return "<b>" + prevDistance + "-" + distance + "m</b>";
                };
                return getTooltipForLineIdx(lineIdx);
            };
            _this._startLTDragHandler = _this._startLTDragHandler.bind(_this);
            _this._stopLTDragHandler = _this._stopLTDragHandler.bind(_this);
            _this._dragLTHandler = _this._dragLTHandler.bind(_this);
            _this._startLTDragPointHandler = _this._startLTDragPointHandler.bind(_this);
            _this._stopLTDragPointHandler = _this._stopLTDragPointHandler.bind(_this);
            _this._dragLTPointHandler = _this._dragLTPointHandler.bind(_this);
            _this._startLTDragCorridorHandler = _this._startLTDragCorridorHandler.bind(_this);
            _this._stopLTDragCorridorHandler = _this._stopLTDragCorridorHandler.bind(_this);
            _this._dragLTCorridorHandler = _this._dragLTCorridorHandler.bind(_this);
            _this._mouseMoveLTLineSplitHandler = _this._mouseMoveLTLineSplitHandler.bind(_this);
            _this.startLTLineSplit = _this.startLTLineSplit.bind(_this);
            _this.stopLTLineSplit = _this.stopLTLineSplit.bind(_this);
            _this.startSelectLTSegmentMode = _this.startSelectLTSegmentMode.bind(_this);
            _this.stopSelectLTSegmentMode = _this.stopSelectLTSegmentMode.bind(_this);
            _this.startRemoveLTPointMode = _this.startRemoveLTPointMode.bind(_this);
            _this.stopRemoveLTPointMode = _this.stopRemoveLTPointMode.bind(_this);
            _this.chooseFirstSegmentToConnect = _this.chooseFirstSegmentToConnect.bind(_this);
            _this.chooseLastSegmentToConnectAndCommit = _this.chooseLastSegmentToConnectAndCommit.bind(_this);
            _this.splitLTByMeters = _this.splitLTByMeters.bind(_this);
            _this._setPointDragStepMode = _this._setPointDragStepMode.bind(_this);
            _this._unsetPointDragStepMode = _this._unsetPointDragStepMode.bind(_this);
            _this._addKeyListener(globals_1.ESC, function () {
                if (_this._LTEditPointIdxTuple) {
                    _this._commitPointDrag();
                    return true;
                }
                else if (_this._lineSplitFn) {
                    _this.stopLTLineSplit();
                    return true;
                }
                else if (_this._selectLTMode) {
                    _this.stopSelectLTSegmentMode();
                    return true;
                }
                return false;
            });
            return _this;
        }
        LajiMapWithLineTransect.prototype.getOptionKeys = function () {
            var _this = this;
            return __assign(__assign({}, _super.prototype.getOptionKeys.call(this)), { lineTransect: ["setLineTransect", function () {
                        return _this.LTFeature ? {
                            feature: _this._formatLTFeatureOut(),
                            activeIdx: _this._LTActiveIdx,
                            onChange: _this._onLTChange
                        } : undefined;
                    }] });
        };
        LajiMapWithLineTransect.prototype._interceptClick = function () {
            if (_super.prototype._interceptClick.call(this))
                return;
            if (this._LTEditPointIdxTuple !== undefined && !this._LTDragging) {
                this._commitPointDrag();
                return true;
            }
            else if (this._lineSplitFn) {
                this._lineSplitFn(this._splitIdxTuple, this._splitPoint);
                return true;
            }
            return false;
        };
        LajiMapWithLineTransect.prototype._getAllData = function () {
            return this._lineLayerGroup ? __spreadArrays(_super.prototype._getAllData.call(this), [{ group: this._corridorLayerGroup }]) : _super.prototype._getAllData.call(this);
        };
        LajiMapWithLineTransect.prototype.setLineTransect = function (data) {
            if (!dependency_utils_1.depsProvided(this, "setLineTransect", arguments))
                return;
            if (!data)
                return;
            var feature = data.feature, activeIdx = data.activeIdx, onChange = data.onChange, getFeatureStyle = data.getFeatureStyle, getTooltip = data.getTooltip, printMode = data.printMode, _a = data.editable, editable = _a === void 0 ? true : _a;
            this.LTFeature = feature;
            this._onLTChange = onChange;
            this._LTActiveIdx = activeIdx;
            this._getLTFeatureStyle = getFeatureStyle;
            this._getLTTooltip = getTooltip;
            this._LTHistory = [{ geometry: feature.geometry }];
            this._LTHistoryPointer = 0;
            if (printMode)
                this._LTPrintMode = true;
            this._LTEditable = this._LTPrintMode ? false : editable;
            this.setLineTransectGeometry(feature.geometry);
            if (this._LTEditable) {
                if (this._origLineTransect) {
                    this._origLineTransect.remove();
                }
                this._origLineTransect = L.featureGroup(this._allSegments.map(function (line) {
                    return L.polyline(line.getLatLngs(), defaultLineStyle).setText("→", {
                        repeat: true,
                        attributes: __assign(__assign({}, defaultLineStyle), { "dy": 5, "font-size": 18 }),
                        below: true
                    });
                })).addTo(this.map).bringToBack();
            }
        };
        LajiMapWithLineTransect.prototype.setLTActiveIdx = function (idx) {
            var _this = this;
            var prevIdx = this._LTActiveIdx;
            this._LTActiveIdx = idx;
            [prevIdx, this._LTActiveIdx].forEach(function (i) { return _this._updateLTStyleForLineIdx(i); });
        };
        LajiMapWithLineTransect.prototype._formatLTFeatureOut = function () {
            var segments = this._lineLayers.map(function (line) { return line.map(function (segment) { return segment.getLatLngs().map(function (_a) {
                var lat = _a.lat, lng = _a.lng;
                return [lng, lat];
            }); }); });
            return __assign(__assign({}, this.LTFeature), { geometry: utils_1.latLngSegmentsToGeoJSONGeometry(segments) });
        };
        LajiMapWithLineTransect.prototype.setLineTransectGeometry = function (geometry, events) {
            var _this = this;
            if (events) {
                if (this._LTHistoryPointer < this._LTHistory.length - 1) {
                    this._LTHistory = this._LTHistory.splice(0).splice(0, this._LTHistoryPointer + 1);
                }
                var undoEvents_1 = [];
                events.forEach(function (e) {
                    switch (e.type) {
                        case "edit": {
                            undoEvents_1.push({
                                type: "edit",
                                idx: e.idx,
                                feature: e.prevFeature,
                                geometry: { type: "LineString", coordinates: e.prevFeature.geometry.coordinates[e.idx] }
                            });
                            break;
                        }
                        case "insert": {
                            undoEvents_1.push({
                                type: "delete",
                                idx: e.idx,
                                feature: e.prevFeature
                            });
                            break;
                        }
                        case "delete": {
                            undoEvents_1.push({
                                type: "insert",
                                idx: e.idx,
                                feature: e.prevFeature,
                                geometry: { type: "LineString", coordinates: e.prevFeature.geometry.coordinates[e.idx] }
                            });
                            break;
                        }
                        case "merge": {
                            undoEvents_1.push({
                                type: "edit",
                                idx: e.idxs[1],
                                feature: e.prevFeature,
                                geometry: { type: "LineString", coordinates: e.prevFeature.geometry.coordinates[e.idxs[1]] }
                            });
                            undoEvents_1.push({
                                type: "insert",
                                idx: e.idxs[0],
                                feature: e.prevFeature,
                                geometry: { type: "LineString", coordinates: e.prevFeature.geometry.coordinates[e.idxs[0]] }
                            });
                            break;
                        }
                        case "move": {
                            undoEvents_1.push({
                                type: "move",
                                idx: e.target,
                                target: e.idx + 1
                            });
                            break;
                        }
                    }
                });
                this._LTHistory.push({ geometry: geometry, undoEvents: undoEvents_1, redoEvents: events });
                this._LTHistoryPointer++;
            }
            var wholeLinesAsSegments = utils_1.geoJSONLineToLatLngSegmentArrays(geometry);
            if (this._pointLayerGroup)
                this.map.removeLayer(this._pointLayerGroup);
            if (this._lineLayerGroup)
                this.map.removeLayer(this._lineLayerGroup);
            if (this._corridorLayerGroup)
                this.map.removeLayer(this._corridorLayerGroup);
            this._pointLayers = [];
            this._lineLayers = [];
            this._corridorLayers = [];
            var pointLayers = this._pointLayers;
            var lineLayers = this._lineLayers;
            var corridorLayers = this._corridorLayers;
            this._overlappingNonadjacentPointIdxTuples = {};
            this._overlappingAdjacentPointIdxTuples = {};
            var overlappingsCoordsToIdxs = {};
            this._pointIdsToIdxTuples = {};
            var indexPoint = function (lat, lng, lineIdx, segmentIdx) {
                var stringCoords = lat + "-" + lng;
                var overlapping = overlappingsCoordsToIdxs[stringCoords];
                if (overlapping) {
                    var overlappingLineIdx = overlapping[0];
                    var pointIdxTuple = [lineIdx, segmentIdx];
                    var pointIdxTupleStr = idxTupleToIdxTupleStr(pointIdxTuple);
                    var overlappingPointIdxTuple = overlappingsCoordsToIdxs[stringCoords];
                    var overlappingPointIdxTupleStr = idxTupleToIdxTupleStr(overlappingPointIdxTuple);
                    if (overlappingLineIdx !== undefined && overlappingLineIdx !== lineIdx - 1) {
                        _this._overlappingNonadjacentPointIdxTuples[pointIdxTupleStr] = overlappingPointIdxTuple;
                        _this._overlappingNonadjacentPointIdxTuples[overlappingPointIdxTupleStr] = pointIdxTuple;
                    }
                    else {
                        _this._overlappingAdjacentPointIdxTuples[pointIdxTupleStr] = overlappingPointIdxTuple;
                        _this._overlappingAdjacentPointIdxTuples[overlappingPointIdxTupleStr] = pointIdxTuple;
                    }
                }
                overlappingsCoordsToIdxs[stringCoords] = [lineIdx, segmentIdx];
            };
            var prevEnd = undefined;
            this._lineIdxsTupleStringsToLineGroupIdxs = {};
            var groupIdx = 0;
            var indexSegment = function (segment, lineIdx) {
                var _a = segment.map(function (c) { return L.latLng(c); }), start = _a[0], end = _a[1];
                if (prevEnd && !start.equals(prevEnd)) {
                    groupIdx++;
                }
                _this._lineIdxsTupleStringsToLineGroupIdxs[lineIdx] = groupIdx;
                prevEnd = end;
            };
            wholeLinesAsSegments.forEach(function (wholeLineAsSegments, lineIdx) {
                [pointLayers, lineLayers, corridorLayers].forEach(function (layers) {
                    layers.push([]);
                });
                var pointLayer = pointLayers[lineIdx];
                var lineLayer = lineLayers[lineIdx];
                var corridorLayer = corridorLayers[lineIdx];
                wholeLineAsSegments.forEach(function (segment, segmentIdx) {
                    var lngLat = segment[0];
                    indexPoint(lngLat[1], lngLat[0], lineIdx, segmentIdx);
                    var line = L.polyline(segment, _this._getStyleForLTIdxTupleAndType([lineIdx, segmentIdx], L.Polyline));
                    if (!_this._LTPrintMode) {
                        line.setText("→", { repeat: true, attributes: { "dy": 5, "font-size": 18 } });
                    }
                    else if (!L.latLng(segment[0]).equals(prevEnd)) {
                        var degree = _this._degreesFromNorth(segment);
                        var direction = L.GeometryUtil.destination(L.latLng(segment[0]), degree, 500);
                        L.polyline([segment[0], direction], { opacity: 0, fillOpacity: 0 })
                            .setText("→", { repeat: false, attributes: { "dy": 5, "font-size": 50 } })
                            .addTo(_this.map);
                    }
                    if (lineIdx === 0 && segmentIdx === 0) {
                        if (_this._LTStartText) {
                            _this._LTStartText.remove();
                        }
                        _this._LTStartText = L.polyline(_this._getLTStartTextCoordinates(segment), { opacity: 0, fillOpacity: 0 })
                            .setText("Alku", { repeat: false, attributes: { "dy": 5, "font-size": 20 } })
                            .addTo(_this.map);
                    }
                    indexSegment(segment, lineIdx);
                    lineLayer.push(line);
                    corridorLayer.push(L.polygon(_this._getCorridorCoordsForLine(segment), _this._getStyleForLTIdxTupleAndType([lineIdx, segmentIdx], L.Polygon)));
                    var point = L.circleMarker(segment[0], _this._getStyleForLTIdxTupleAndType([lineIdx, segmentIdx], L.CircleMarker));
                    pointLayer.push(point);
                    _this._pointIdsToIdxTuples[L.Util.stamp(point)] = [lineIdx, segmentIdx];
                    if (segmentIdx === wholeLineAsSegments.length - 1) {
                        var _lngLat = segment[1];
                        indexPoint(_lngLat[1], _lngLat[0], lineIdx, segmentIdx + 1);
                        var _point = L.circleMarker(_lngLat, _this._getStyleForLTIdxTupleAndType([lineIdx, segmentIdx + 1], L.CircleMarker));
                        pointLayer.push(_point);
                        _this._pointIdsToIdxTuples[L.Util.stamp(_point)] = [lineIdx, segmentIdx + 1];
                    }
                });
            });
            this._allSegments = flattenMatrix(lineLayers);
            this._allCorridors = flattenMatrix(corridorLayers);
            this._allPoints = flattenMatrix(pointLayers);
            this._lineLayerGroup = L.featureGroup(this._allSegments).addTo(this.map);
            this._corridorLayerGroup = L.featureGroup(this._allCorridors).addTo(this.map);
            this._pointLayerGroup = L.featureGroup(this._allPoints).addTo(this.map);
            this._LTGroups = this._lineLayers.map(function (_, lineIdx) {
                return L.featureGroup(__spreadArrays(_this._lineLayers[lineIdx], _this._pointLayers[lineIdx], _this._corridorLayers[lineIdx]));
            });
            this._groupIdxsToLineIdxs = Object.keys(this._lineIdxsTupleStringsToLineGroupIdxs).reduce(function (d, _lineIdx) {
                var group = d[_this._lineIdxsTupleStringsToLineGroupIdxs[_lineIdx]] || [];
                d[_this._lineIdxsTupleStringsToLineGroupIdxs[_lineIdx]] = group;
                group.push(+_lineIdx);
                return d;
            }, {});
            this._setIdxTupleMappings();
            this._setLineTransectEvents();
            this._setLTPrintLines();
            dependency_utils_1.provide(this, "lineTransect");
        };
        LajiMapWithLineTransect.prototype._getLTStartTextCoordinates = function (segment) {
            segment = segment.map(L.latLng);
            var degree = this._degreesFromNorth(segment);
            var length = degree > -50 && degree < 50 ? 120 : 60;
            return [
                segment[0],
                L.GeometryUtil.destination(segment[0], 90, 500)
            ].map(function (c) { return L.GeometryUtil.destination(c, degree - 90, length); });
        };
        LajiMapWithLineTransect.prototype.LTUndo = function () {
            if (this._LTHistoryPointer <= 0)
                return;
            var undoEvents = this._LTHistory[this._LTHistoryPointer].undoEvents;
            this._LTHistoryPointer--;
            var geometry = this._LTHistory[this._LTHistoryPointer].geometry;
            this.setLineTransectGeometry(geometry);
            if (undoEvents) {
                this._triggerEvent(undoEvents.slice(0).reverse(), this._onLTChange);
            }
        };
        LajiMapWithLineTransect.prototype.LTRedo = function () {
            if (this._LTHistoryPointer >= this._LTHistory.length - 1)
                return;
            this._LTHistoryPointer++;
            var _a = this._LTHistory[this._LTHistoryPointer], geometry = _a.geometry, redoEvents = _a.redoEvents;
            this.setLineTransectGeometry(geometry);
            if (redoEvents) {
                this._triggerEvent(redoEvents, this._onLTChange);
            }
        };
        LajiMapWithLineTransect.prototype._clearTooltipDescription = function () {
            this._updateLTTooltip();
            this._tooltipIdx = undefined;
        };
        LajiMapWithLineTransect.prototype.flatIdxToIdxTuple = function (idx) {
            var lineIdx = 0;
            var line = this._pointLayers[lineIdx];
            while (idx >= line.length) {
                idx -= line.length;
                lineIdx++;
                line = this._pointLayers[lineIdx];
            }
            return [lineIdx, idx];
        };
        // Opens a dialog and asks which point to use, if points are overlapping.
        LajiMapWithLineTransect.prototype._getPoint = function (idxTuple, callback, questionTranslationKey, firstTranslationKey, lastTranslationKey) {
            var _this = this;
            if (questionTranslationKey === void 0) { questionTranslationKey = "FirstOrLastPoint"; }
            if (firstTranslationKey === void 0) { firstTranslationKey = "FirstPartitive"; }
            if (lastTranslationKey === void 0) { lastTranslationKey = "LastPartitive"; }
            var lineIdx = idxTuple[0], pointIdx = idxTuple[1];
            var overlappingPointIdxTuple = this._overlappingNonadjacentPointIdxTuples[idxTupleToIdxTupleStr([lineIdx, pointIdx])];
            var latLng = this._getLTLayerForIdxTuple(this._pointLayers, [lineIdx, pointIdx]).getLatLng();
            var _latLng = overlappingPointIdxTuple
                ? this._getLTLayerForIdxTuple(this._pointLayers, overlappingPointIdxTuple).getLatLng()
                : undefined;
            if (overlappingPointIdxTuple !== undefined && latLng.equals(_latLng)) {
                var firstIdxTuple = overlappingPointIdxTuple;
                var lastIdxTuple = [lineIdx, pointIdx];
                var lastPoint_1 = this._getLTLayerForIdxTuple(this._pointLayers, lastIdxTuple);
                var translateHooks_1 = [];
                var popup = document.createElement("div");
                popup.className = "text-center";
                var question = document.createElement("span");
                translateHooks_1.push(this.addTranslationHook(question, questionTranslationKey));
                var precedingIdxTuple = this._getIdxTuplePrecedingPoint(firstIdxTuple);
                var followingIdxTuple = this._getIdxTupleForLineFollowingPoint([lineIdx, pointIdx]);
                var onClick = function (_idxTuple) { return function (e) {
                    e.preventDefault();
                    var point = _this._getLTLayerForIdxTuple(_this._pointLayers, _idxTuple);
                    _this._overlappingPointDialogSegmentIdxTuple = undefined;
                    point.setStyle(pointStyle);
                    lastPoint_1.closePopup();
                    callback(_idxTuple);
                }; };
                var onMouseOver = function (_idxTuple) { return function () {
                    if (!_idxTuple)
                        return;
                    _this._overlappingPointDialogSegmentIdxTuple = _idxTuple;
                    _this._updateLTStyleForIdxTuple(_idxTuple);
                }; };
                var onMouseOut = function (_idxTuple) { return function () {
                    if (!_idxTuple)
                        return;
                    _this._overlappingPointDialogSegmentIdxTuple = undefined;
                    _this._updateLTStyleForIdxTuple(_idxTuple);
                }; };
                var firstButton = document.createElement("button");
                firstButton.addEventListener("click", onClick(firstIdxTuple));
                firstButton.addEventListener("mouseover", onMouseOver(precedingIdxTuple));
                firstButton.addEventListener("mouseout", onMouseOut(precedingIdxTuple));
                translateHooks_1.push(this.addTranslationHook(firstButton, firstTranslationKey));
                var lastButton = document.createElement("button");
                lastButton.addEventListener("click", onClick(lastIdxTuple));
                lastButton.addEventListener("mouseover", onMouseOver(followingIdxTuple));
                lastButton.addEventListener("mouseout", onMouseOut(followingIdxTuple));
                translateHooks_1.push(this.addTranslationHook(lastButton, lastTranslationKey));
                var buttonContainer_1 = document.createElement("div");
                buttonContainer_1.className = "btn-group";
                [firstButton, lastButton].forEach(function (button) {
                    button.className = "btn btn-primary btn-xs";
                    buttonContainer_1.appendChild(button);
                });
                popup.appendChild(question);
                popup.appendChild(buttonContainer_1);
                lastPoint_1.bindPopup(popup).openPopup();
                lastPoint_1.on("popupclose", function () {
                    translateHooks_1.forEach(function (hook) { return _this.removeTranslationHook(hook); });
                    lastPoint_1.unbindPopup();
                });
            }
            else {
                callback(idxTuple);
            }
        };
        LajiMapWithLineTransect.prototype.getIdxsFromLayer = function (layer) {
            var _this = this;
            if (!layer)
                return undefined;
            var getIdxsForId = function (id) {
                var lineIdx = _this.leafletIdsToCorridorLineIdxs[id];
                var segmentIdx = _this.leafletIdsToCorridorSegmentIdxs[id];
                return {
                    i: _this.leafletIdsToFlatCorridorSegmentIdxs[id],
                    lineIdx: lineIdx,
                    segmentIdx: segmentIdx,
                    idxTuple: [lineIdx, segmentIdx]
                };
            };
            var id = L.Util.stamp(layer);
            if (layer instanceof L.CircleMarker) {
                var i = this.leafletIdsToFlatPointIdxs[id];
                var _a = this.flatIdxToIdxTuple(i), lineIdx = _a[0], segmentIdx = _a[1];
                return {
                    i: i,
                    lineIdx: lineIdx,
                    segmentIdx: segmentIdx,
                    idxTuple: [lineIdx, segmentIdx]
                };
            }
            else if (map_1.isPolyline(layer)) {
                var corridorId = this.lineIdsToCorridorIds[L.Util.stamp(layer)];
                return getIdxsForId(corridorId);
            }
            else {
                return getIdxsForId(id);
            }
        };
        LajiMapWithLineTransect.prototype.getIdxsFromEvent = function (_a) {
            var layer = _a.layer;
            return this.getIdxsFromLayer(layer);
        };
        // Handles also distance calculation
        LajiMapWithLineTransect.prototype._setIdxTupleMappings = function () {
            var _this = this;
            this.leafletIdsToFlatCorridorSegmentIdxs = {};
            this.leafletIdsToCorridorLineIdxs = {};
            this.leafletIdsToCorridorSegmentIdxs = {};
            this.corridorFlatIdxsToLeafletIds = {};
            this.lineIdsToCorridorIds = {};
            var i = 0;
            this._corridorLayers.forEach(function (corridors, lineIdx) { return corridors.forEach(function (corridor, segmentIdx) {
                var id = L.Util.stamp(corridor);
                _this.leafletIdsToFlatCorridorSegmentIdxs[id] = i;
                _this.leafletIdsToCorridorLineIdxs[id] = lineIdx;
                _this.leafletIdsToCorridorSegmentIdxs[id] = segmentIdx;
                _this.corridorFlatIdxsToLeafletIds[i] = id;
                i++;
            }); });
            i = 0;
            this._lineLayers.forEach(function (lines) { return lines.forEach(function (line) {
                _this.lineIdsToCorridorIds[L.Util.stamp(line)] = _this.corridorFlatIdxsToLeafletIds[i];
                i++;
            }); });
            this.leafletIdsToFlatPointIdxs = {};
            i = 0;
            this._pointLayers.forEach(function (points) {
                points.forEach(function (point) {
                    _this.leafletIdsToFlatPointIdxs[L.Util.stamp(point)] = i;
                    i++;
                });
            });
        };
        LajiMapWithLineTransect.prototype._setLineTransectEvents = function () {
            var _this = this;
            var onMouseOver = function (e) {
                L.DomEvent.stopPropagation(e);
                var isPoint = e.layer instanceof L.Marker;
                var _a = _this.getIdxsFromEvent(e), lineIdx = _a.lineIdx, segmentIdx = _a.segmentIdx;
                var prevHoverIdx = _this._hoveredIdxTuple;
                _this._hoveredIdxTuple = [lineIdx, segmentIdx];
                _this._hoveredType = isPoint ? "point" : "corridor";
                if (prevHoverIdx)
                    _this._updateLTStyleForLineIdx(prevHoverIdx[0]);
                _this._updateLTStyleForLineIdx(_this._hoveredIdxTuple[0]);
                _this._updateLTTooltip();
            };
            var onMouseOut = function (e) {
                L.DomEvent.stopPropagation(e);
                var lineIdx = _this.getIdxsFromEvent(e).lineIdx;
                _this._hoveredType = undefined;
                _this._hoveredIdxTuple = undefined;
                _this._updateLTStyleForLineIdx(lineIdx);
                _this._updateLTTooltip();
            };
            var pointIsMiddlePoint = function (e) {
                var _a = _this.getIdxsFromEvent(e), lineIdx = _a.lineIdx, segmentIdx = _a.segmentIdx;
                if (segmentIdx === 0 || segmentIdx === _this._pointLayers[lineIdx].length - 1) {
                    return false;
                }
                return true;
            };
            var delayClick = function (fn) {
                if (_this._LTClickTimeout)
                    clearTimeout(_this._LTClickTimeout);
                if (_this._closebyPointIdxTuple) {
                    _this._LTClickTimeout = setTimeout(fn, 500);
                }
                else {
                    fn();
                }
            };
            this._pointLayerGroup.on("dblclick", function (e) {
                L.DomEvent.stopPropagation(e);
                clearTimeout(_this._LTClickTimeout);
                var idxTuple = _this.getIdxsFromEvent(e).idxTuple;
                _this._getPoint(idxTuple, function (_idxTuple) { return _this._setLTPointEditable(_idxTuple); });
            }).on("click", function (e) {
                L.DomEvent.stopPropagation(e);
                var _a = _this.getIdxsFromEvent(e), lineIdx = _a.lineIdx, segmentIdx = _a.segmentIdx, idxTuple = _a.idxTuple;
                if (_this._pointLTShiftMode && _this._pointCanBeShiftedTo(idxTuple)) {
                    _this.commitLTPointShift([segmentIdx === 0 ? lineIdx : lineIdx + 1, 0]);
                    return;
                }
                _this._interceptClick();
                var idxTupleStr = idxTuple ? idxTupleToIdxTupleStr(idxTuple) : undefined;
                if (_this._overlappingNonadjacentPointIdxTuples[idxTupleStr]
                    || _this._overlappingAdjacentPointIdxTuples[idxTupleStr]) {
                    return;
                }
                delayClick(function () {
                    var _lineIdx = _this.getIdxsFromEvent(e).lineIdx;
                    if (!_this._selectLTMode) {
                        _this._triggerEvent(_this._getOnActiveSegmentChangeEvent(_lineIdx), _this._onLTChange);
                        _this._updateLTTooltip();
                    }
                });
            }).on("mouseover", function (e) {
                pointIsMiddlePoint(e) && onMouseOver(e);
            }).on("mouseout", function (e) {
                pointIsMiddlePoint(e) && onMouseOut(e);
            });
            this._corridorLayerGroup.on("click", function (e) {
                L.DomEvent.stopPropagation(e);
                delayClick(function () {
                    if (_this._interceptClick())
                        return;
                    var _a = _this.getIdxsFromEvent(e), lineIdx = _a.lineIdx, idxTuple = _a.idxTuple;
                    if (_this._closebyPointIdxTuple && _this._pointLTShiftMode && _this._pointCanBeShiftedTo(_this._closebyPointIdxTuple)) {
                        var _b = _this._closebyPointIdxTuple, closebyLineIdx = _b[0], closebySegmentIdx = _b[1];
                        _this.commitLTPointShift([closebySegmentIdx === 0 ? closebyLineIdx : closebyLineIdx + 1, 0]);
                        return;
                    }
                    if (_this._selectLTMode) {
                        _this._hoveredIdxTuple = undefined;
                        if (_this._onSelectLT)
                            _this._onSelectLT(idxTuple);
                    }
                    else {
                        _this._triggerEvent(_this._getOnActiveSegmentChangeEvent(lineIdx), _this._onLTChange);
                        _this._updateLTTooltip();
                    }
                });
            }).on("mouseover", onMouseOver)
                .on("mouseout", onMouseOut);
            this.map.on("mousemove", function (_a) {
                var latlng = _a.latlng;
                if (_this._splitIdxTuple
                    || _this._firstLTSegmentToRemoveIdx
                    || _this._selectLTMode
                    || _this.map.contextmenu.isVisible()) {
                    return;
                }
                var closestPoint = L.GeometryUtil.closestLayer(_this.map, _this._allPoints, latlng).layer;
                var idxTuple = _this.getIdxsFromLayer(closestPoint).idxTuple;
                var idxTupleStr = idxTuple ? idxTupleToIdxTupleStr(idxTuple) : undefined;
                var prevClosestPointIdxTuple = _this._closebyPointIdxTuple;
                var closestPointPixelPoint = _this.map.latLngToLayerPoint(closestPoint.getLatLng());
                var latLngPixelPoint = _this.map.latLngToLayerPoint(latlng);
                _this._closebyPointIdxTuple = _this._contextMenuLayer !== undefined && _this._contextMenuLayer === _this._LTPointExpander
                    ? _this._closebyPointIdxTuple
                    : closestPointPixelPoint.distanceTo(latLngPixelPoint) <= POINT_DIST_TRESHOLD
                        ? idxTuple
                        : undefined;
                if (!idxTuplesEqual(prevClosestPointIdxTuple, _this._closebyPointIdxTuple)) {
                    if (_this._LTPointExpander) {
                        var layer = _this._LTPointExpander;
                        layer.remove();
                        _this._updateContextMenu && _this._updateContextMenu();
                    }
                    if (_this._closebyPointIdxTuple) {
                        _this._LTPointExpander = new L.CircleMarker(closestPoint.getLatLng(), {
                            radius: POINT_DIST_TRESHOLD,
                            opacity: 0,
                            fillOpacity: 0
                        })
                            .addTo(_this.map)
                            .bringToBack();
                        _this.map.contextmenu.removeAllItems();
                        _this._getContextMenuForPoint(_this._closebyPointIdxTuple).contextmenuItems
                            .forEach(function (item) { return _this.map.contextmenu.addItem(item); });
                        var layer = _this._getLTLayerForIdxTuple(_this._pointLayers, _this._closebyPointIdxTuple);
                        if (layer && _this.map.hasLayer(layer))
                            layer.bringToFront();
                        if (_this._LTdragPoint)
                            _this._LTdragPoint.bringToFront();
                    }
                    [prevClosestPointIdxTuple, _this._closebyPointIdxTuple].forEach(function (_idxTuple) {
                        if (!_idxTuple)
                            return;
                        var layers = _this._layerExistsForIdxTuple(_this._pointLayers, _idxTuple)
                            ? [_this._getLTLayerForIdxTuple(_this._pointLayers, _idxTuple)]
                            : [];
                        var overlappingNonadjacentIdxTuple = _this._overlappingNonadjacentPointIdxTuples[idxTupleStr];
                        var overlappingAdjacentIdxTuple = _this._overlappingAdjacentPointIdxTuples[idxTupleStr];
                        if (overlappingNonadjacentIdxTuple) {
                            layers.push(_this._getLTLayerForIdxTuple(_this._pointLayers, overlappingNonadjacentIdxTuple));
                        }
                        else if (overlappingAdjacentIdxTuple) {
                            layers.push(_this._getLTLayerForIdxTuple(_this._pointLayers, overlappingAdjacentIdxTuple));
                        }
                        layers.forEach(function (layer) { return layer && _this._setStyleForLTLayer(layer); });
                    });
                    _this._updateLTTooltip();
                }
            }).on("click", function (e) {
                L.DomEvent.stopPropagation(e);
                if (_this._closebyPointIdxTuple && _this._pointLTShiftMode) {
                    _this._getPoint(_this._closebyPointIdxTuple, function (idxTuple) { return _this.commitLTPointShift(idxTuple); });
                }
            }).on("dblclick", function (e) {
                L.DomEvent.stopPropagation(e);
                if (_this._closebyPointIdxTuple) {
                    clearTimeout(_this._LTClickTimeout);
                    _this._disableDblClickZoom = true;
                    _this._getPoint(_this._closebyPointIdxTuple, function (idxTuple) { return _this._setLTPointEditable(idxTuple); });
                    setTimeout(function () {
                        _this._disableDblClickZoom = false;
                    }, 10);
                }
            }).on("contextmenu.show", function (e) {
                if (e.relatedTarget)
                    _this._LTContextMenuLayer = e.relatedTarget;
            }).on("contextmenu.hide", function () {
                var lineIdx = (_this.getIdxsFromLayer(_this._LTContextMenuLayer) || {}).lineIdx;
                if (lineIdx !== undefined)
                    _this._updateLTStyleForLineIdx(lineIdx);
            }).on("controlClick", function () {
                if (_this._LTEditPointIdxTuple) {
                    _this._commitPointDrag();
                }
            });
        };
        LajiMapWithLineTransect.prototype._updateLTLayerContextMenus = function () {
            var _this = this;
            if (!dependency_utils_1.depsProvided(this, "_updateLTLayerContextMenus", arguments))
                return;
            this._pointLayers.forEach(function (points, lineIdx) { return points.forEach(function (point, pointIdx) {
                point.bindContextMenu(_this._getContextMenuForPoint([lineIdx, pointIdx]));
            }); });
        };
        LajiMapWithLineTransect.prototype._getContextMenuForPoint = function (idxTuple) {
            var _this = this;
            var lineIdx = idxTuple[0], pointIdx = idxTuple[1];
            if (!this._LTEditable)
                return { contextmenuItems: [] };
            var contextmenuItems = [
                {
                    text: this.translations.RemovePoint,
                    callback: function () {
                        _this._getPoint([lineIdx, pointIdx], function (_idxTuple) { return _this.removeLTPoint(_idxTuple); }, "RemoveFirstOrLastPoint", "First", "Last");
                    },
                    iconCls: "glyphicon glyphicon-remove-sign"
                },
                {
                    text: this.translations.EditPoint,
                    callback: function () {
                        _this._getPoint([lineIdx, pointIdx], function (_idxTuple) { return _this._setLTPointEditable(_idxTuple); }, "RemoveFirstOrLastPoint", "First", "Last");
                    },
                    iconCls: "glyphicon glyphicon-remove-sign"
                }
            ];
            if (this._pointCanBeShiftedTo([lineIdx, pointIdx])) {
                contextmenuItems.push({
                    text: this.translations.ShiftPoint,
                    callback: function () { return _this.commitLTPointShift([lineIdx, pointIdx]); },
                    iconCls: "laji-map-line-transect-shift-point-glyph"
                });
            }
            return {
                contextmenuInheritItems: false,
                contextmenuItems: contextmenuItems
            };
        };
        // @param 'commit' can be an array of events that are triggered
        // at the same time as the event that this function triggers.
        LajiMapWithLineTransect.prototype.removeLTPoint = function (idxTuple, commit) {
            if (commit === void 0) { commit = true; }
            var lineIdx = idxTuple[0];
            this._commitPointDrag();
            var events = [];
            var that = this;
            var prevFeature = this._formatLTFeatureOut();
            var precedingIdxTuple = this._getIdxTuplePrecedingPoint(idxTuple);
            var followingIdxTuple = this._getIdxTupleForLineFollowingPoint(idxTuple);
            var _a = precedingIdxTuple || [undefined, undefined], precedingLineIdx = _a[0], precedingSegmentIdx = _a[1];
            var _b = followingIdxTuple || [undefined, undefined], followingLineIdx = _b[0], followingSegmentIdx = _b[1];
            var precedingSegment = precedingIdxTuple ? this._getLTLayerForIdxTuple(this._lineLayers, precedingIdxTuple) : undefined;
            var followingSegment = followingIdxTuple ? this._getLTLayerForIdxTuple(this._lineLayers, followingIdxTuple) : undefined;
            var precedingLine = this._lineLayers[precedingLineIdx];
            var followingLine = this._lineLayers[followingLineIdx];
            if (precedingLine === followingLine) {
                var start = precedingSegment.getLatLngs()[0];
                var end = followingSegment.getLatLngs()[1];
                precedingSegment.setLatLngs([start, end]);
                this._lineLayers[precedingLineIdx] = precedingLine.filter(function (l) { return l !== followingSegment; });
                events = addMiddlePointRemoveEvent(events);
            }
            else if (precedingLine && !followingLine) {
                precedingLine.splice(precedingSegmentIdx, 1);
                events = addMiddlePointRemoveEvent(events);
            }
            else if (!precedingLine && followingLine) {
                followingLine.splice(followingSegmentIdx, 1);
                events = addMiddlePointRemoveEvent(events);
            }
            else if (precedingLine && followingLine) {
                var _precedingSegment = precedingLine[precedingSegmentIdx];
                var _followingSegment = followingLine[followingSegmentIdx];
                _precedingSegment.setLatLngs([
                    _precedingSegment.getLatLngs()[0],
                    _followingSegment.getLatLngs()[1]
                ]);
                followingLine.splice(followingSegmentIdx, 1);
                this._lineLayers[precedingLineIdx] = __spreadArrays(precedingLine, followingLine);
                this._lineLayers.splice(followingLineIdx, 1);
                var feature = this._formatLTFeatureOut();
                events = [
                    {
                        type: "merge",
                        idxs: [precedingLineIdx, followingLineIdx],
                        feature: feature,
                        geometry: { type: "LineString", coordinates: feature.geometry.coordinates[precedingLineIdx] },
                        prevFeature: prevFeature
                    }
                ];
            }
            if (this._LTActiveIdx !== undefined && this._LTActiveIdx > lineIdx) {
                this._LTActiveIdx = this._LTActiveIdx - 1;
            }
            if (commit) {
                if (Array.isArray(commit)) {
                    events = __spreadArrays(commit, events);
                }
                this.setLineTransectGeometry(this._formatLTFeatureOut().geometry, events);
                this._triggerEvent(events, this._onLTChange);
            }
            else {
                return events;
            }
            function addMiddlePointRemoveEvent(_events) {
                var feature = that._formatLTFeatureOut();
                return __spreadArrays(_events, [{
                        type: "edit",
                        idx: precedingLineIdx,
                        feature: feature,
                        geometry: { type: "LineString", coordinates: feature.geometry.coordinates[precedingLineIdx] },
                        prevFeature: prevFeature
                    }]);
            }
        };
        LajiMapWithLineTransect.prototype._setLTPointEditable = function (idxTuple) {
            var _this = this;
            var lineIdx = idxTuple[0], pointIdx = idxTuple[1];
            if (!this._LTEditable || this._pointLTShiftMode)
                return;
            if (idxTuplesEqual(this._LTEditPointIdxTuple, [lineIdx, pointIdx]))
                return;
            if (this._LTEditPointIdxTuple !== undefined) {
                var _a = this._LTEditPointIdxTuple, _lineIdx = _a[0], _segmentIdx = _a[1];
                var editableLayer = this._pointLayers[_lineIdx][_segmentIdx];
                editableLayer.setStyle(pointStyle);
                this._commitPointDrag();
            }
            var overlappingSeamPointIdx = this._overlappingAdjacentPointIdxTuples[idxTupleToIdxTupleStr([lineIdx, pointIdx])];
            if (overlappingSeamPointIdx) {
                var overlappingPoint = this._getLTLayerForIdxTuple(this._pointLayers, overlappingSeamPointIdx);
                overlappingPoint.remove();
            }
            this._LTEditPointIdxTuple = [lineIdx, pointIdx];
            this._featureBeforePointDrag = this._formatLTFeatureOut();
            if (pointIdx === undefined) {
                return;
            }
            var point = this._getLTLayerForIdxTuple(this._pointLayers, [lineIdx, pointIdx]);
            this._LTPointLatLngBeforeDrag = point.getLatLng();
            var style = { color: "#ff0000", opacity: 0.5, fillColor: "#ffffff", fillOpacity: 0.3 };
            this._LTdragPoint = new L.CircleMarker(point.getLatLng(), __assign({ radius: POINT_DIST_TRESHOLD }, style));
            this._LTdragPoint.addTo(this.map)
                .bringToFront()
                .on("mouseover", function () {
                _this._hoveringDragPoint = true;
                _this._LTdragPoint.setStyle(style);
                _this._setStyleForLTLayer(point);
                _this._updateLTTooltip();
            })
                .on("mouseout", function () {
                _this._hoveringDragPoint = false;
                _this._LTdragPoint.setStyle(__assign(__assign({}, style), { opacity: 0.3 }));
                _this._setStyleForLTLayer(point);
                _this._updateLTTooltip();
            })
                .on("remove", function () { return _this._setStyleForLTLayer(point); })
                .on("mousedown", this._startLTDragPointHandler)
                .on("mouseup", this._stopLTDragPointHandler);
            [pointIdx, pointIdx - 1].filter(function (i) { return i >= 0; }).forEach(function (idx) {
                var corridor = _this._corridorLayers[lineIdx][idx];
                if (corridor)
                    corridor.on("mousedown", _this._startLTDragCorridorHandler);
            });
            this.map.on("mouseup", this._stopLTDragCorridorHandler);
            this._clearTooltipDescription();
            this._setStyleForLTLayer(point);
            [
                this._getIdxTuplePrecedingEditPoint(),
                this._getIdxTupleFollowingEditPoint()
            ].filter(function (i) { return i; })
                .map(function (_idxTuple) { return [_this._lineLayers, _this._corridorLayers].map(function (layers) {
                return layers === _this._lineLayers
                    ? _this._getLTLayerForIdxTuple(layers, _idxTuple)
                    : _this._getLTLayerForIdxTuple(layers, _idxTuple);
            }); })
                .forEach(function (layerPair) { return layerPair.forEach(function (layer) { return _this._setStyleForLTLayer(layer); }); });
            this._addKeyListener("control", this._setPointDragStepMode);
            this._addKeyListener("control", this._unsetPointDragStepMode, "keyup");
            this._updateLTTooltip();
        };
        LajiMapWithLineTransect.prototype._commitPointDrag = function () {
            var _this = this;
            if (!this._LTEditPointIdxTuple)
                return;
            this._stopLTDragPointHandler();
            var precedingIdxTuple = this._getIdxTuplePrecedingEditPoint();
            var followingIdxTuple = this._getIdxTupleFollowingEditPoint();
            this._LTEditPointIdxTuple = undefined;
            this._updateLTTooltip();
            var dragPointLatLng = this._LTdragPoint.getLatLng();
            this._LTdragPoint.remove();
            this._LTdragPoint = undefined;
            this._removeKeyListener("control", this._setPointDragStepMode);
            this._removeKeyListener("control", this._unsetPointDragStepMode, "keyup");
            [precedingIdxTuple, followingIdxTuple].forEach(function (tuple) {
                if (tuple) {
                    _this._getLTLayerForIdxTuple(_this._corridorLayers, tuple).off("mousedown").off("mouseup");
                    _this._updateLTStyleForIdxTuple(tuple);
                }
            });
            if (this._LTPointLatLngBeforeDrag.equals(dragPointLatLng)) {
                return;
            }
            var feature = this._formatLTFeatureOut();
            var events = [];
            var prevLineIdx = undefined;
            [precedingIdxTuple, followingIdxTuple].forEach(function (idxTuple) {
                if (!idxTuple)
                    return;
                var lineIdx = idxTuple[0];
                if (lineIdx !== undefined && lineIdx !== prevLineIdx) {
                    prevLineIdx = lineIdx;
                    events.push({
                        type: "edit",
                        feature: feature,
                        prevFeature: _this._featureBeforePointDrag,
                        idx: lineIdx,
                        geometry: lineToGeoJSONLine(_this._lineLayers[lineIdx])
                    });
                }
            });
            this.setLineTransectGeometry(feature.geometry, events);
            this._triggerEvent(events, this._onLTChange);
            this.map.fire("lineTransect:pointdrag");
            this._updateLTTooltip();
        };
        LajiMapWithLineTransect.prototype._setPointDragStepMode = function () {
            this._pointDragSnapMode = true;
            this._updateLTTooltip();
        };
        LajiMapWithLineTransect.prototype._unsetPointDragStepMode = function () {
            this._pointDragSnapMode = false;
            this._updateLTTooltip();
        };
        LajiMapWithLineTransect.prototype._startLTDragHandler = function (handler) {
            this._LTDragging = true;
            this.map.dragging.disable();
            L.DomUtil.disableTextSelection();
            this.map.on("mousemove", handler);
        };
        LajiMapWithLineTransect.prototype._stopLTDragHandler = function (handler) {
            var _this = this;
            // _interceptClick is triggered after mouseup - we delay drag stopping until map click is handled.
            setTimeout(function () {
                _this._LTDragging = false;
                _this.map.dragging.enable();
                L.DomUtil.enableTextSelection();
                _this.map.off("mousemove", handler);
                _this._updateLTTooltip();
            });
        };
        LajiMapWithLineTransect.prototype._startLTDragPointHandler = function (_a) {
            var latlng = _a.latlng;
            var _b = this._LTEditPointIdxTuple, lineIdx = _b[0], pointIdx = _b[1];
            var point = this._pointLayers[lineIdx][pointIdx];
            this._dragPointStart = point.getLatLng();
            this._dragMouseStart = latlng;
            this._startLTDragHandler(this._dragLTPointHandler);
        };
        LajiMapWithLineTransect.prototype._stopLTDragPointHandler = function () {
            this._stopLTDragHandler(this._dragLTPointHandler);
        };
        LajiMapWithLineTransect.prototype._dragLTPointHandler = function (_a) {
            var latlng = _a.latlng;
            if (!this._dragMouseStart)
                return;
            this._updateLTTooltip();
            var mouseMovedDistance = this._dragMouseStart.distanceTo(latlng);
            var mouseRotatedAngle = this._degreesFromNorth([this._dragMouseStart, latlng]);
            var offsetDragPoint = L.GeometryUtil.destination(this._dragPointStart, mouseRotatedAngle, mouseMovedDistance);
            this._dragLTHandler(offsetDragPoint);
        };
        LajiMapWithLineTransect.prototype._startLTDragCorridorHandler = function (e) {
            var latlng = e.latlng;
            this._startLTDragHandler(this._dragLTCorridorHandler);
            var _a = this._LTEditPointIdxTuple, lineIdx = _a[0], pointIdx = _a[1];
            var point = this._pointLayers[lineIdx][pointIdx];
            this._dragCorridor = e.target;
            this._dragPointStart = point.getLatLng();
            this._dragMouseStart = latlng;
        };
        LajiMapWithLineTransect.prototype._stopLTDragCorridorHandler = function () {
            this._stopLTDragHandler(this._dragLTCorridorHandler);
            this._dragPointStart = undefined;
            this._dragMouseStart = undefined;
            this._dragCorridor = undefined;
        };
        LajiMapWithLineTransect.prototype._dragLTCorridorHandler = function (_a) {
            var latlng = _a.latlng;
            if (!this._dragMouseStart)
                return;
            var mouseMovedDistance = this._dragMouseStart.distanceTo(latlng);
            var mouseRotatedAngle = this._degreesFromNorth([this._dragMouseStart, latlng]);
            var offsetDragPoint = L.GeometryUtil.destination(this._dragPointStart, mouseRotatedAngle, mouseMovedDistance);
            this._dragLTHandler(offsetDragPoint);
        };
        LajiMapWithLineTransect.prototype._dragLTHandler = function (latlng) {
            var _this = this;
            var snapped = false;
            var useSnap = function (idxTuple) {
                if (snapped) {
                    return;
                }
                snapped = true;
                var computeFromPoint = _this._getLTLayerForIdxTuple(_this._pointLayers, idxTuple);
                var points = [computeFromPoint.getLatLng(), latlng];
                var angle = _this._degreesFromNorth(points);
                var snappedAngle = Math.round(angle / 45) * 45;
                return L.GeometryUtil.destination(computeFromPoint.getLatLng(), snappedAngle, computeFromPoint.getLatLng().distanceTo(latlng));
            };
            var idxs = this._LTEditPointIdxTuple;
            var lineIdx = idxs[0], pointIdx = idxs[1];
            var pointLayer = this._pointLayers[lineIdx];
            var point = pointLayer[pointIdx];
            var precedingIdxTuple = this._getIdxTuplePrecedingEditPoint();
            var precedingLine, precedingCorridor;
            if (precedingIdxTuple) {
                precedingLine = this._getLTLayerForIdxTuple(this._lineLayers, precedingIdxTuple);
                precedingCorridor = this._getLTLayerForIdxTuple(this._corridorLayers, precedingIdxTuple);
                if (this._pointDragSnapMode && (!this._dragCorridor || precedingCorridor === this._dragCorridor)) {
                    latlng = useSnap(precedingIdxTuple);
                }
            }
            var followingIdxTuple = this._getIdxTupleFollowingEditPoint();
            var followingLine, followingCorridor;
            if (followingIdxTuple) {
                followingLine = this._getLTLayerForIdxTuple(this._lineLayers, followingIdxTuple);
                followingCorridor = this._getLTLayerForIdxTuple(this._corridorLayers, followingIdxTuple);
            }
            var idxTupleFollowingFollowing = followingIdxTuple && this._getIdxTupleForFollowingPoint(followingIdxTuple);
            var followingFollowingCorridor = idxTupleFollowingFollowing && this._getLTLayerForIdxTuple(this._corridorLayers, followingIdxTuple);
            if (this._pointDragSnapMode && (!precedingIdxTuple || this._dragCorridor === followingFollowingCorridor)) {
                var snappedLatLng = useSnap(idxTupleFollowingFollowing);
                if (snappedLatLng) {
                    latlng = snappedLatLng;
                }
            }
            if (this._LTPointIdxTupleIsGroupFirstOrLast(idxs)) {
                var pointExcluded = this._allPoints.filter(function (p) { return p !== point; });
                var closestPoint = L.GeometryUtil.closestLayer(this.map, this._allPoints.filter(function (p) { return p !== point; }), latlng).layer;
                var closestIdxTuple = this._pointIdsToIdxTuples[L.Util.stamp(closestPoint)];
                if (closestIdxTuple
                    && this._LTPointIdxTupleIsGroupFirstOrLast(closestIdxTuple)
                    && this._LTIdxTuplesAreFromSameGroup(idxs, closestIdxTuple)) {
                    var precedingToClosest = this._getIdxTuplePrecedingPoint(closestIdxTuple);
                    var followingToClosest = this._getIdxTupleForLineFollowingPoint(closestIdxTuple);
                    var closestPointPixelPoint = this.map.latLngToLayerPoint(closestPoint.getLatLng());
                    var latLngPixelPoint = this.map.latLngToLayerPoint(latlng);
                    if ((!precedingToClosest || !followingToClosest) && closestPointPixelPoint.distanceTo(latLngPixelPoint) <= 20) {
                        latlng = closestPoint.getLatLng();
                    }
                }
            }
            if (precedingIdxTuple) {
                var lineCoords = [precedingLine.getLatLngs()[0], latlng];
                precedingLine.setLatLngs(lineCoords);
                precedingCorridor.setLatLngs(this._getCorridorCoordsForLine(lineCoords));
            }
            var getFollowingLineCoords = function () { return [latlng, followingLine.getLatLngs()[1]]; };
            if (followingIdxTuple) {
                var lineCoords = getFollowingLineCoords();
                followingLine.setLatLngs(lineCoords);
                followingCorridor.setLatLngs(this._getCorridorCoordsForLine(lineCoords));
            }
            point.setLatLng(latlng);
            this._LTdragPoint.setLatLng(latlng);
            if (lineIdx === 0 && pointIdx === 0) {
                this._LTStartText.setLatLngs(this._getLTStartTextCoordinates(getFollowingLineCoords()));
            }
        };
        LajiMapWithLineTransect.prototype._getLTLayerForIdxTuple = function (layers, idxTuple) {
            var lineIdx = idxTuple[0], segmentIdx = idxTuple[1];
            return layers[lineIdx][segmentIdx];
        };
        LajiMapWithLineTransect.prototype._layerExistsForIdxTuple = function (layer, idxTuple) {
            var lineIdx = idxTuple[0], segmentIdx = idxTuple[1];
            return layer && layer[lineIdx] && layer[lineIdx][segmentIdx];
        };
        LajiMapWithLineTransect.prototype._getIdxTuplePrecedingPoint = function (idxTuple) {
            var lineIdx = idxTuple[0], pointIdx = idxTuple[1];
            if (lineIdx === undefined || pointIdx === undefined)
                return undefined;
            var pointLayer = this._pointLayers[lineIdx];
            var point = pointLayer[pointIdx];
            var precedingLineIdx, precedingIdx = undefined;
            if (pointIdx - 1 >= 0) {
                precedingLineIdx = lineIdx;
                precedingIdx = pointIdx - 1;
            }
            else if (lineIdx - 1 >= 0) {
                var precedingLineLayer = this._lineLayers[lineIdx - 1];
                var endLatLng = precedingLineLayer[precedingLineLayer.length - 1].getLatLngs()[1];
                if (endLatLng.equals(point.getLatLng())) {
                    precedingLineIdx = lineIdx - 1;
                    precedingIdx = precedingLineLayer.length - 1;
                }
            }
            return precedingLineIdx !== undefined && precedingIdx !== undefined ? [precedingLineIdx, precedingIdx] : undefined;
        };
        LajiMapWithLineTransect.prototype._getIdxTuplePrecedingEditPoint = function () {
            return this._LTEditPointIdxTuple
                ? this._getIdxTuplePrecedingPoint(this._LTEditPointIdxTuple)
                : undefined;
        };
        LajiMapWithLineTransect.prototype._getIdxTupleForLineFollowingPoint = function (idxTuple) {
            var lineIdx = idxTuple[0], pointIdx = idxTuple[1];
            if (lineIdx === undefined || pointIdx === undefined)
                return undefined;
            var pointLayer = this._pointLayers[lineIdx];
            var point = pointLayer[pointIdx];
            var followingLineIdx, followingIdx = undefined;
            if (pointIdx < pointLayer.length - 1) {
                followingLineIdx = lineIdx;
                followingIdx = pointIdx;
            }
            else if (lineIdx + 1 <= this._lineLayers.length - 1) {
                var followingLineLayer = this._lineLayers[lineIdx + 1];
                var startLatLng = followingLineLayer[0].getLatLngs()[0];
                if (startLatLng.equals(point.getLatLng())) {
                    followingLineIdx = lineIdx + 1;
                    followingIdx = 0;
                }
            }
            return followingLineIdx !== undefined && followingIdx !== undefined
                ? [followingLineIdx, followingIdx]
                : undefined;
        };
        LajiMapWithLineTransect.prototype._getIdxTupleFollowingEditPoint = function () {
            return this._LTEditPointIdxTuple
                ? this._getIdxTupleForLineFollowingPoint(this._LTEditPointIdxTuple)
                : undefined;
        };
        LajiMapWithLineTransect.prototype._getIdxTupleForFollowingPoint = function (idxTuple) {
            var lineIdx = idxTuple[0], pointIdx = idxTuple[1];
            if (lineIdx === undefined || pointIdx === undefined)
                return undefined;
            var pointLayer = this._pointLayers[lineIdx];
            var point = pointLayer[pointIdx];
            var followingLineIdx, followingIdx = undefined;
            if (pointIdx < pointLayer.length - 1) {
                followingLineIdx = lineIdx;
                followingIdx = pointIdx + 1;
            }
            else if (lineIdx + 1 <= this._lineLayers.length - 1) {
                var followingLineLayer = this._lineLayers[lineIdx + 1];
                var startLatLng = followingLineLayer[0].getLatLngs()[0];
                if (startLatLng.equals(point.getLatLng())) {
                    followingLineIdx = lineIdx + 1;
                    followingIdx = 0;
                }
            }
            return followingLineIdx !== undefined && followingIdx !== undefined
                ? [followingLineIdx, followingIdx]
                : undefined;
        };
        LajiMapWithLineTransect.prototype._degreesFromNorth = function (lineCoords) {
            var latLngs = lineCoords.map(L.latLng).map(function (latlng) { return L.CRS.EPSG3857.project(latlng); });
            // Line angle horizontally.
            var lineAngle = L.GeometryUtil.computeAngle(latLngs[0], latLngs[1]);
            // Line angle clockwise from north.
            return 90 - lineAngle;
        };
        LajiMapWithLineTransect.prototype._getCorridorCoordsForLine = function (lineCoords) {
            var latLngs = lineCoords.map(L.latLng);
            var lineAngleFromNorth = this._degreesFromNorth(lineCoords);
            // Variables are named as if the line was pointing towards north.
            var SWCorner = L.GeometryUtil.destination(latLngs[0], lineAngleFromNorth - 90, LT_WIDTH_METERS);
            var NWCorner = L.GeometryUtil.destination(latLngs[1], lineAngleFromNorth - 90, LT_WIDTH_METERS);
            var SECorner = L.GeometryUtil.destination(latLngs[0], lineAngleFromNorth + 90, LT_WIDTH_METERS);
            var NECorner = L.GeometryUtil.destination(latLngs[1], lineAngleFromNorth + 90, LT_WIDTH_METERS);
            return [SWCorner, NWCorner, NECorner, SECorner];
        };
        LajiMapWithLineTransect.prototype._getOnActiveSegmentChangeEvent = function (lineIdx) {
            var _this = this;
            var prevIdx = this._LTActiveIdx;
            this._LTActiveIdx = lineIdx;
            [prevIdx, lineIdx].forEach(function (i) { return _this._updateLTStyleForLineIdx(i); });
            return { type: "active", idx: this._LTActiveIdx };
        };
        LajiMapWithLineTransect.prototype._getStyleForLTIdxTupleAndType = function (idxTuple, type) {
            var lineIdx = idxTuple[0], segmentIdx = idxTuple[1];
            var isPoint = type === L.CircleMarker;
            var isActive = lineIdx === this._LTActiveIdx
                && (!isPoint || (segmentIdx !== 0 && segmentIdx !== this._pointLayers[lineIdx].length - 1));
            var _a = this._hoveredIdxTuple || [undefined, undefined], hoveredLineIdx = _a[0], hoveredSegmentIdx = _a[1];
            var contextMenuLineIdx = (this.getIdxsFromLayer(this._contextMenuLayer)
                || {}).lineIdx;
            var isEditPoint = isPoint && idxTuplesEqual(idxTuple, this._LTEditPointIdxTuple);
            var isHintPoint = isPoint && this._pointLTShiftMode && this._pointCanBeShiftedTo(idxTuple);
            var isClosebyPoint = isPoint
                && idxTuplesEqual(idxTuple, this._closebyPointIdxTuple)
                && (!this._pointLTShiftMode || (isHintPoint));
            var isFirstOverlappingEndOrStartPoint = isPoint && ((!this._overlappingNonadjacentPointIdxTuples["0-0"] && idxTuplesEqual(idxTuple, [0, 0])) ||
                (this._overlappingNonadjacentPointIdxTuples["0-0"]
                    && (Object.keys(this._overlappingNonadjacentPointIdxTuples)[0] === idxTupleToIdxTupleStr(idxTuple)
                        || idxTuplesEqual([0, 0], idxTuple))));
            var isOverlappingEndOrStartPoint = isPoint &&
                !isFirstOverlappingEndOrStartPoint &&
                this._overlappingNonadjacentPointIdxTuples.hasOwnProperty(idxTupleToIdxTupleStr(idxTuple));
            var isSeamPoint = isPoint
                && this._overlappingAdjacentPointIdxTuples.hasOwnProperty(idxTupleToIdxTupleStr(idxTuple));
            var _isHover = lineIdx === hoveredLineIdx || lineIdx === contextMenuLineIdx;
            var isEdit = isPoint
                ? isEditPoint
                : idxTuplesEqual(idxTuple, this._splitIdxTuple)
                    || idxTuplesEqual(idxTuple, this._firstLTSegmentToRemoveIdx)
                    || idxTuplesEqual(idxTuple, this._overlappingPointDialogSegmentIdxTuple)
                    || idxTuplesEqual(idxTuple, this._getIdxTuplePrecedingEditPoint())
                    || idxTuplesEqual(idxTuple, this._getIdxTupleFollowingEditPoint())
                    || (this._selectLTMode === "segment" && _isHover && segmentIdx === hoveredSegmentIdx)
                    || (this._selectLTMode === "line" && _isHover);
            var isHover = this._splitIdxTuple || this._firstLTSegmentToRemoveIdx || this._selectLTMode
                ? false
                : isPoint
                    ? !isSeamPoint && !isOverlappingEndOrStartPoint && _isHover && !isActive
                    : _isHover && !isActive;
            function createPrintStylesFor(styles) {
                return Object.keys(styles).reduce(function (o, key) {
                    o[key] = { opacity: 0, fillOpacity: 0 };
                    return o;
                }, {});
            }
            var lineStyles = {
                normal: lineStyle,
                odd: lineStyle,
                active: activeLineStyle,
                edit: editLineStyle,
                hover: hoverLineStyle
            };
            lineStyles.print = createPrintStylesFor(lineStyles);
            lineStyles.print.normal = { weight: 1, color: "#000" };
            lineStyles.print.odd = { weight: 1, color: "#000" };
            var corridorStyles = {
                normal: corridorStyle,
                odd: oddCorridorStyle,
                active: activeCorridorStyle,
                edit: editCorridorStyle,
                hover: hoverCorridorStyle
            };
            corridorStyles.print = createPrintStylesFor(corridorStyles);
            var pointStyles = {
                normal: pointStyle,
                odd: oddPointStyle,
                active: activePointStyle,
                edit: editPointStyle,
                editPoint: editablePointStyle,
                hover: hoverPointStyle,
                closebyEdit: closebyEditPointStyle,
                closeby: closebyPointStyle,
                hint: hintPointStyle,
                seam: seamPointStyle,
                overlappingSeam: overlappingPointStyle,
                firstOverlappingSeam: firstOverlappingPointStyle
            };
            pointStyles.print = createPrintStylesFor(pointStyles);
            pointStyles.print.firstOverlappingSeam = __assign(__assign({}, firstOverlappingPointStyle), { weight: 0 });
            pointStyles.print.overlappingSeam = __assign(__assign({}, overlappingPointStyle), { fillColor: "#f77", weight: 0 });
            var styleObject = undefined;
            if (type === L.Polygon) {
                styleObject = corridorStyles;
            }
            else if (type === L.Polyline) {
                styleObject = lineStyles;
            }
            else if (type === L.CircleMarker) {
                styleObject = pointStyles;
            }
            if (this._LTPrintMode) {
                styleObject = styleObject.print;
            }
            if (isEditPoint && isClosebyPoint && this._LTEditable) {
                return styleObject.closebyEdit;
            }
            else if (isClosebyPoint && this._LTEditable) {
                return styleObject.closeby;
            }
            else if (isHintPoint) {
                return styleObject.hint;
            }
            else if (isEditPoint) {
                return styleObject.editPoint;
            }
            else if (isFirstOverlappingEndOrStartPoint) {
                return styleObject.firstOverlappingSeam;
            }
            else if (isOverlappingEndOrStartPoint) {
                return styleObject.overlappingSeam;
            }
            else if (isSeamPoint) {
                return styleObject.seam;
            }
            else if (isEdit && this._LTEditable) {
                return styleObject.edit;
            }
            else if (isHover && !this._LTPrintMode) {
                return styleObject.hover;
            }
            else if (isActive && !this._LTPrintMode) {
                return styleObject.active;
            }
            else {
                if (this._getLTFeatureStyle) {
                    var style = this._getLTFeatureStyle({ lineIdx: lineIdx, segmentIdx: segmentIdx, type: type, style: styleObject.normal });
                    if (style)
                        return style;
                }
                return lineIdx % 2 === 0 ? styleObject.normal : styleObject.odd;
            }
        };
        LajiMapWithLineTransect.prototype._setStyleForLTLayer = function (layer) {
            var _a = this.getIdxsFromLayer(layer), lineIdx = _a.lineIdx, segmentIdx = _a.segmentIdx;
            layer.setStyle(this._getStyleForLTIdxTupleAndType([lineIdx, segmentIdx], layer.constructor));
        };
        LajiMapWithLineTransect.prototype._updateLTStyleForLineIdx = function (lineIdx) {
            var _this = this;
            if (lineIdx === undefined)
                return;
            this._corridorLayers[lineIdx].forEach(function (corridorLayer) {
                var segmentIdx = _this.getIdxsFromLayer(corridorLayer).segmentIdx;
                _this._updateLTStyleForIdxTuple([lineIdx, segmentIdx]);
            });
        };
        LajiMapWithLineTransect.prototype._updateLTStyleForIdxTuple = function (idxTuple) {
            var _this = this;
            var lineIdx = idxTuple[0], segmentIdx = idxTuple[1];
            if (this._LTPrintMode || lineIdx === undefined || segmentIdx === undefined)
                return;
            [this._lineLayers, this._corridorLayers, this._pointLayers].forEach(function (layerGroup) {
                if (layerGroup === _this._pointLayers && segmentIdx === 0
                    || (_this._pointLayers[lineIdx] && segmentIdx === _this._pointLayers[lineIdx].length - 1))
                    return;
                var lineGroup = layerGroup[lineIdx] || [];
                var layer = lineGroup[segmentIdx];
                if (layer)
                    _this._setStyleForLTLayer(layer);
            });
        };
        LajiMapWithLineTransect.prototype._idxTupleToFlatIdx = function (idxTuple) {
            var lineIdx = idxTuple[0], segmentIdx = idxTuple[1];
            if (lineIdx === undefined || segmentIdx === undefined)
                return undefined;
            return this.getIdxsFromLayer(this._lineLayers[lineIdx][segmentIdx]).i;
        };
        LajiMapWithLineTransect.prototype._commitLTLineSplit = function (idxTuple, splitPoint) {
            this.stopLTLineSplit();
            var prevFeature = this._formatLTFeatureOut();
            var lineIdx = idxTuple[0], segmentIdx = idxTuple[1];
            var splitLine = this._lineLayers[lineIdx][segmentIdx];
            var _a = splitLine.getLatLngs(), start = _a[0], end = _a[1];
            // Tail is the part prepending the split and head the following part.
            var splittedSegmentTail = [start, splitPoint];
            var splittedSegmentHead = [splitPoint, end];
            splitLine.setLatLngs(splittedSegmentTail);
            this._lineLayers[lineIdx].splice(segmentIdx + 1, 0, L.polyline(splittedSegmentHead));
            var splittedLineTail = this._lineLayers[lineIdx].slice(0, segmentIdx + 1);
            var splittedLineHead = this._lineLayers[lineIdx].slice(segmentIdx + 1);
            this._lineLayers[lineIdx] = splittedLineTail;
            this._lineLayers.splice(lineIdx + 1, 0, splittedLineHead);
            var feature = this._formatLTFeatureOut();
            var events = [
                {
                    type: "edit",
                    feature: feature,
                    prevFeature: prevFeature,
                    idx: lineIdx,
                    geometry: lineToGeoJSONLine(splittedLineTail)
                },
                {
                    type: "insert",
                    idx: lineIdx + 1,
                    geometry: lineToGeoJSONLine(splittedLineHead),
                    prevFeature: prevFeature,
                }
            ];
            this.setLineTransectGeometry(feature.geometry, events);
            if (lineIdx < this._LTActiveIdx) {
                events.push(this._getOnActiveSegmentChangeEvent(this._LTActiveIdx + 1));
            }
            this._triggerEvent(events, this._onLTChange);
            this.map.fire("lineTransect:split");
        };
        LajiMapWithLineTransect.prototype._commitLTPointAdd = function (idxTuple, splitPoint) {
            this.stopLTLineSplit();
            var prevFeature = this._formatLTFeatureOut();
            var lineIdx = idxTuple[0], segmentIdx = idxTuple[1];
            var splitLine = this._lineLayers[lineIdx][segmentIdx];
            var _a = splitLine.getLatLngs(), start = _a[0], end = _a[1];
            // Tail is the part prepending the split and head the following part.
            var splittedSegmentTail = [start, splitPoint];
            var splittedSegmentHead = [splitPoint, end];
            splitLine.setLatLngs(splittedSegmentTail);
            this._lineLayers[lineIdx].splice(segmentIdx + 1, 0, L.polyline(splittedSegmentHead));
            var feature = this._formatLTFeatureOut();
            var events = [
                {
                    type: "edit",
                    feature: feature,
                    idx: lineIdx,
                    geometry: lineToGeoJSONLine(this._lineLayers[lineIdx]),
                    prevFeature: prevFeature
                },
            ];
            this.setLineTransectGeometry(feature.geometry, events);
            this._triggerEvent(events, this._onLTChange);
            this.map.fire("lineTransect:pointadd");
        };
        LajiMapWithLineTransect.prototype.stopLTLineSplit = function () {
            var lastLineCutIdx = this._splitIdxTuple;
            this._lineSplitFn = undefined;
            if (this._cutLine)
                this._cutLine.removeFrom(this.map);
            this._cutLine = undefined;
            this._lineCutIdx = undefined;
            this._splitIdxTuple = undefined;
            this.map.off("mousemove", this._mouseMoveLTLineSplitHandler);
            if (lastLineCutIdx)
                this._updateLTStyleForIdxTuple(lastLineCutIdx);
            this._disposeTooltip();
        };
        LajiMapWithLineTransect.prototype._mouseMoveLTLineSplitHandler = function (_a) {
            var latlng = _a.latlng;
            var allSegments = this._allSegments;
            var closestLine, closestIdx;
            if (this._lineCutIdx !== undefined) {
                var _b = this._lineCutIdx, lineIdx = _b[0], segmentIdx = _b[1];
                closestIdx = this._lineCutIdx;
                closestLine = this._lineLayers[lineIdx][segmentIdx];
            }
            else {
                closestLine = L.GeometryUtil.closestLayer(this.map, allSegments, latlng).layer;
                closestIdx = this.getIdxsFromLayer(closestLine).idxTuple;
            }
            var prevCutIdx = this._splitIdxTuple;
            this._splitIdxTuple = closestIdx;
            if (prevCutIdx)
                this._updateLTStyleForIdxTuple(prevCutIdx);
            if (this._splitIdxTuple)
                this._updateLTStyleForIdxTuple(this._splitIdxTuple);
            // Update cut line.
            var closestLatLngOnLine = L.GeometryUtil.closest(this.map, closestLine, latlng);
            this._splitPoint = closestLatLngOnLine;
            var lineAngleFromNorth = this._degreesFromNorth(closestLine.getLatLngs());
            var cutLineStart = L.GeometryUtil.destination(closestLatLngOnLine, lineAngleFromNorth - 90, LT_WIDTH_METERS);
            var cutLineEnd = L.GeometryUtil.destination(closestLatLngOnLine, lineAngleFromNorth + 90, LT_WIDTH_METERS);
            if (this._cutLine) {
                this._cutLine.setLatLngs([cutLineStart, cutLineEnd]);
            }
            else {
                this._cutLine = L.polygon([cutLineStart, cutLineEnd], __assign(__assign({}, editLineStyle), { dashArray: "5 5" })).addTo(this.map);
            }
        };
        LajiMapWithLineTransect.prototype.startLTLineSplit = function () {
            this._lineSplitFn = this._commitLTLineSplit;
            this.map.on("mousemove", this._mouseMoveLTLineSplitHandler);
            this._mouseMoveLTLineSplitHandler({ latlng: this._mouseLatLng });
            this._createTooltip("SplitLineTooltip");
        };
        LajiMapWithLineTransect.prototype.startLTLineSplitForIdx = function (idxTuple) {
            this._lineCutIdx = idxTuple;
            this._lineSplitFn = this._commitLTLineSplit;
            this.map.on("mousemove", this._mouseMoveLTLineSplitHandler);
            this._mouseMoveLTLineSplitHandler({ latlng: this._mouseLatLng });
            this._createTooltip("SplitLineTooltip");
        };
        LajiMapWithLineTransect.prototype.startLTPointAdd = function () {
            this._lineSplitFn = this._commitLTPointAdd;
            this.map.on("mousemove", this._mouseMoveLTLineSplitHandler);
            this._mouseMoveLTLineSplitHandler({ latlng: this._mouseLatLng });
            this._createTooltip("AddPointTooltip");
        };
        LajiMapWithLineTransect.prototype.startLTPointAddSplitForIdx = function (idxTuple) {
            this._lineCutIdx = idxTuple;
            this._lineSplitFn = this._commitLTPointAdd;
            this.map.on("mousemove", this._mouseMoveLTLineSplitHandler);
            this._mouseMoveLTLineSplitHandler({ latlng: this._mouseLatLng });
            this._createTooltip("SplitLineTooltip");
        };
        LajiMapWithLineTransect.prototype._LTPointIdxTupleIsGroupFirstOrLast = function (idxTuple) {
            var lineIdx = idxTuple[0], segmentIdx = idxTuple[1];
            var groupIdx = this._lineIdxsTupleStringsToLineGroupIdxs[lineIdx];
            var group = this._groupIdxsToLineIdxs[groupIdx];
            return !!(lineIdx === group[0] && segmentIdx === 0
                || lineIdx === group[group.length - 1] && segmentIdx === this._pointLayers[lineIdx].length - 1);
        };
        LajiMapWithLineTransect.prototype._LTIdxTuplesAreFromSameGroup = function (idxTuple, idxTuple2) {
            var lineIdx = idxTuple[0], segmentIdx = idxTuple[1];
            var lineIdx2 = idxTuple2[0], segmentIdx2 = idxTuple2[1];
            return this._lineIdxsTupleStringsToLineGroupIdxs[lineIdx] === this._lineIdxsTupleStringsToLineGroupIdxs[lineIdx2];
        };
        LajiMapWithLineTransect.prototype._pointCanBeShiftedTo = function (idxTuple) {
            var lineIdx = idxTuple[0], pointIdx = idxTuple[1];
            return this._lineIdxsTupleStringsToLineGroupIdxs[lineIdx] === 0
                && (pointIdx === 0 || pointIdx === this._pointLayers[lineIdx].length - 1);
        };
        LajiMapWithLineTransect.prototype.startLTPointShift = function () {
            this._pointLTShiftMode = true;
            for (var _i = 0, _a = Object.keys(this._lineIdxsTupleStringsToLineGroupIdxs); _i < _a.length; _i++) {
                var _lineIdx = _a[_i];
                var lineIdx = parseInt(_lineIdx);
                var groupIdx = this._lineIdxsTupleStringsToLineGroupIdxs[lineIdx];
                if (groupIdx === 0) {
                    this._setStyleForLTLayer(this._getLTLayerForIdxTuple(this._pointLayers, [lineIdx, 0]));
                    this._setStyleForLTLayer(this._getLTLayerForIdxTuple(this._pointLayers, [lineIdx, this._pointLayers[lineIdx].length - 1]));
                }
            }
            this._createTooltip("ShiftPointTooltip");
        };
        LajiMapWithLineTransect.prototype.stopLTPointShift = function () {
            this._pointLTShiftMode = false;
            for (var _i = 0, _a = Object.keys(this._lineIdxsTupleStringsToLineGroupIdxs); _i < _a.length; _i++) {
                var _lineIdx = _a[_i];
                var lineIdx = parseInt(_lineIdx);
                var groupIdx = this._lineIdxsTupleStringsToLineGroupIdxs[lineIdx];
                if (groupIdx === 0) {
                    this._setStyleForLTLayer(this._getLTLayerForIdxTuple(this._pointLayers, [lineIdx, 0]));
                    this._setStyleForLTLayer(this._getLTLayerForIdxTuple(this._pointLayers, [lineIdx, this._pointLayers[lineIdx].length - 1]));
                }
            }
            this._disposeTooltip();
        };
        LajiMapWithLineTransect.prototype.commitLTPointShift = function (idxTuple) {
            var _lineIdx = idxTuple[0], pointIdx = idxTuple[1];
            this.stopLTPointShift();
            var lineIdx = pointIdx === 0
                ? _lineIdx
                : pointIdx === this._pointLayers[_lineIdx].length - 1
                    ? _lineIdx + 1
                    : _lineIdx;
            var prevEnd = undefined;
            var idx = 0;
            for (var _idx in this._lineLayers) {
                idx = parseInt(_idx);
                var line = this._lineLayers[idx];
                var start = line[0].getLatLngs()[0];
                var end = line[line.length - 1].getLatLngs()[1];
                if (prevEnd && !start.equals(prevEnd)) {
                    break;
                }
                else if (idx === this._lineLayers.length - 1) {
                    idx = undefined;
                }
                prevEnd = end;
            }
            var connectedLines = this._lineLayers.slice(0, idx);
            var disconnectedLines = idx !== undefined
                ? this._lineLayers.slice(idx, this._lineLayers.length)
                : [];
            var headLines = connectedLines.slice(0, lineIdx);
            var tailLines = connectedLines.slice(lineIdx, connectedLines.length);
            var prevFeature = this._formatLTFeatureOut();
            this._lineLayers = __spreadArrays(tailLines, headLines, disconnectedLines);
            var events = [];
            for (var i = lineIdx + tailLines.length - 1; i >= lineIdx; i--) {
                events.push({
                    type: "move",
                    idx: lineIdx + tailLines.length - 1,
                    target: 0,
                    prevFeature: prevFeature
                });
            }
            this.setLineTransectGeometry(this._formatLTFeatureOut().geometry, events);
            this._triggerEvent(events, this._onLTChange);
        };
        LajiMapWithLineTransect.prototype.startSelectLTSegmentMode = function (onSelect, tooltip, mode) {
            var _this = this;
            if (mode === void 0) { mode = "segment"; }
            this._selectLTMode = mode;
            this._onSelectLT = function (idxTuple) {
                if (onSelect(idxTuple) !== false)
                    _this.stopSelectLTSegmentMode(idxTuple);
            };
            if (tooltip)
                this._createTooltip(tooltip);
        };
        LajiMapWithLineTransect.prototype.stopSelectLTSegmentMode = function (idxTuple) {
            if (idxTuple === void 0) { idxTuple = [undefined, undefined]; }
            var lineIdx = idxTuple[0], segmentIdx = idxTuple[1];
            this._selectLTMode = undefined;
            this._onSelectLT = undefined;
            if (this._hoveredIdxTuple)
                this._updateLTStyleForIdxTuple(this._hoveredIdxTuple);
            if (lineIdx !== undefined && segmentIdx !== undefined)
                this._updateLTStyleForIdxTuple([lineIdx, segmentIdx]);
            this._disposeTooltip();
        };
        LajiMapWithLineTransect.prototype.startRemoveLTPointMode = function () {
            this.startSelectLTSegmentMode(this.chooseFirstSegmentToConnect, "startLineConnectFirstPointHelp");
        };
        LajiMapWithLineTransect.prototype.stopRemoveLTPointMode = function () {
            var _idxTuple = this._firstLTSegmentToRemoveIdx;
            this._firstLTSegmentToRemoveIdx = undefined;
            if (_idxTuple)
                this._updateLTStyleForIdxTuple(_idxTuple);
            this.stopSelectLTSegmentMode();
        };
        LajiMapWithLineTransect.prototype.chooseFirstSegmentToConnect = function (idxTuple) {
            this._firstLTSegmentToRemoveIdx = idxTuple;
            this._updateLTStyleForIdxTuple(idxTuple);
            this.startSelectLTSegmentMode(this.chooseLastSegmentToConnectAndCommit, "startLineConnectLastPointHelp");
            return false;
        };
        LajiMapWithLineTransect.prototype.chooseLastSegmentToConnectAndCommit = function (idxTuple) {
            var _this = this;
            var _a = [this._firstLTSegmentToRemoveIdx, idxTuple]
                .map(function (tuple) { return _this._idxTupleToFlatIdx(tuple); }).sort(function (a, b) { return a - b; }), first = _a[0], last = _a[1];
            var timeout = undefined;
            var prevLatLng = undefined;
            for (var i_1 = first; i_1 <= last; i_1++) {
                var segment = this._allSegments[i_1];
                if (!prevLatLng) {
                    prevLatLng = segment.getLatLngs()[1];
                    continue;
                }
                if (!segment.getLatLngs()[0].equals(prevLatLng)) {
                    this._createTooltip("SegmentsMustBeOfSameLine", !!"error");
                    if (timeout)
                        clearTimeout(timeout);
                    timeout = setTimeout(function () {
                        if (!_this._firstLTSegmentToRemoveIdx)
                            return;
                        _this._createTooltip("startLineConnectLastPointHelp");
                        timeout = undefined;
                    }, 2000);
                    return false;
                }
                prevLatLng = segment.getLatLngs()[1];
            }
            this._firstLTSegmentToRemoveIdx = undefined;
            var flatIdxToIdxTuple = function (idx) {
                return (idx === undefined) ? undefined : _this.getIdxsFromLayer(_this._allSegments[idx]).idxTuple;
            };
            var i = last;
            var events = [];
            while (i !== first) {
                var _events = this.removeLTPoint(flatIdxToIdxTuple(i), i === first + 1 ? events : false);
                if (_events) {
                    events = __spreadArrays(events, _events);
                }
                i--;
            }
        };
        LajiMapWithLineTransect.prototype.splitLTByMeters = function () {
            var _this = this;
            var splitByMeters = function (e) {
                e.preventDefault();
                var value = parseInt(input.value);
                var distance = 0;
                var distanceLessThanLength;
                var currentSegmentIdx = 0;
                var currentLineIdx = 0;
                var currentSegment;
                while (true) {
                    var currentLine = _this._lineLayers[currentLineIdx];
                    currentSegment = currentLine[currentSegmentIdx];
                    var _a = currentSegment.getLatLngs(), start_1 = _a[0], end = _a[1];
                    distanceLessThanLength = distance;
                    distance += start_1.distanceTo(end);
                    if (distance >= value)
                        break;
                    if (currentSegmentIdx >= currentLine.length - 1) {
                        currentSegmentIdx = 0;
                        currentLineIdx++;
                    }
                    else {
                        currentSegmentIdx++;
                    }
                }
                var remainingLength = value - distanceLessThanLength;
                var lineAngleFromNorth = _this._degreesFromNorth(currentSegment.getLatLngs());
                var splitPoint = L.GeometryUtil.destination(currentSegment.getLatLngs()[0], lineAngleFromNorth, remainingLength);
                _this._commitLTLineSplit([currentLineIdx, currentSegmentIdx], splitPoint);
                if (_this._selectLTMode)
                    _this.stopSelectLTSegmentMode();
                _this._closeDialog(e);
            };
            var translateHooks = [];
            var container = document.createElement("form");
            var feature = this._formatLTFeatureOut();
            var _a = utils_1.getLineTransectStartEndDistancesForIdx(feature, 99999), start = _a[0], length = _a[1]; // eslint-disable-line no-unused-vars
            var help = document.createElement("span");
            help.className = "help-block";
            translateHooks.push(this.addTranslationHook(help, function () { return _this.translations.SegmentSplitByLengthHelp + ": " + length + "m"; }));
            var input = utils_1.createTextInput();
            input.className += " form-group";
            var prevVal = "";
            input.oninput = function (e) {
                var target = e.target;
                target.value = target.value.replace(",", ".");
                if (!target.value.match(/^\d*\.?\d*$/)) {
                    target.value = prevVal;
                }
                prevVal = target.value;
                if (target.value === "" || parseInt(target.value) < 0 || parseInt(target.value) > length) {
                    submit.setAttribute("disabled", "disabled");
                }
                else {
                    submit.removeAttribute("disabled");
                }
            };
            var submit = document.createElement("button");
            submit.setAttribute("type", "submit");
            submit.className = "btn btn-block btn-primary";
            translateHooks.push(this.addTranslationHook(submit, "SplitWholeLine"));
            submit.setAttribute("disabled", "disabled");
            submit.addEventListener("click", splitByMeters);
            container.appendChild(help);
            container.appendChild(input);
            container.appendChild(submit);
            this._showDialog(container, function () {
                translateHooks.forEach(function (hook) { return _this.removeTranslationHook(hook); });
                submit.removeEventListener("click", splitByMeters);
            });
            input.focus();
        };
        // Computes the messages to display.
        LajiMapWithLineTransect.prototype._updateLTTooltip = function () {
            var _this = this;
            if (!this._LTEditable)
                return;
            var messages = {
                text: undefined,
                click: undefined,
                dblclick: undefined,
                rightclick: undefined,
                drag: undefined
            };
            var hoveredIdxTuple = this._hoveredIdxTuple || (this._hoveringDragPoint && this._LTEditPointIdxTuple);
            if (hoveredIdxTuple) {
                messages.text = this._getTooltipForPointIdxTuple(this._hoveredIdxTuple ? L.Marker : L.Polygon, hoveredIdxTuple);
            }
            if (this._LTEditPointIdxTuple) {
                if (this._hoveringDragPoint || this._hoveredType === "corridor" || this._dragPointStart) {
                    messages.drag = this.translations.toMovePoint;
                }
                if (!this._hoveringDragPoint && this._hoveredType !== "corridor") {
                    messages.click = this.translations.toCommitEdit;
                }
                if (!this._pointDragSnapMode) {
                    messages.text = "" + (messages.text || "") + (messages.text ? "<br>" : "") + this.translations.EnterSnapMode;
                }
            }
            else if (this._hoveredIdxTuple) {
                messages.click = this.translations.toActivate;
            }
            if (this._closebyPointIdxTuple) {
                messages.dblclick = this.translations.toEditPoint;
                messages.rightclick = this.translations.toDeletePoint;
            }
            this.__updateLTTooltip(messages);
            if (hoveredIdxTuple && this._getLTTooltip) {
                var lineIdx_1 = hoveredIdxTuple[0];
                this._tooltipIdx = lineIdx_1;
                var result = this._getLTTooltip(lineIdx_1, messages.text, function (callbackText) {
                    if (_this._tooltipIdx === lineIdx_1)
                        _this.__updateLTTooltip({ text: callbackText });
                });
                if (result !== undefined && typeof result !== "function") {
                    this.__updateLTTooltip({ text: result });
                }
            }
        };
        // Displays the given messages.
        LajiMapWithLineTransect.prototype.__updateLTTooltip = function (messages) {
            var _this = this;
            var message = "";
            if (this._tooltip && this._tooltip !== this._ltTooltip)
                return;
            this.messages = __assign(__assign({}, this.messages), messages);
            var order = ["text", "drag", "click", "dblclick", "rightclick"];
            Object.keys(this.messages)
                .sort(function (a, b) { return order.indexOf(a) - order.indexOf(b); })
                .forEach(function (key) {
                if (_this.messages[key]) {
                    var prefix = message ? "<br />" : "";
                    var actionTranslation = _this.translations[utils_1.capitalizeFirstLetter(key)];
                    var actionText = actionTranslation ? "<b>" + actionTranslation + "</b> " : "";
                    message += "" + prefix + actionText + _this.messages[key];
                }
            });
            if (message && !this._ltTooltip) {
                this._ltTooltip = this._createTooltip(message);
            }
            else if (message) {
                this._ltTooltip.updateContent({ text: message });
            }
            else {
                this._disposeTooltip();
                this._ltTooltip = undefined;
            }
        };
        LajiMapWithLineTransect.prototype._setLTPrintLines = function () {
            var _this = this;
            if (!this._LTPrintMode) {
                return;
            }
            var nonusedDist = 0;
            var counter = 0;
            this._allSegments.forEach(function (segment, i) {
                var lengths = i === 0 ? [0] : [];
                var _a = segment.getLatLngs(), start = _a[0], end = _a[1];
                var segmentLength = start.distanceTo(end);
                var _segmentLength = segmentLength;
                while (_segmentLength + nonusedDist > 100) {
                    var length_1 = 100 - nonusedDist;
                    _segmentLength -= length_1;
                    lengths.push(length_1);
                    nonusedDist = 0;
                }
                nonusedDist = lengths.reduce(function (total, l) { return total - l; }, segmentLength + nonusedDist);
                var lineAngleFromNorth = _this._degreesFromNorth(segment.getLatLngs());
                var accumulatedLength = 0;
                lengths.forEach(function (length) {
                    var major = counter === 0 || counter % 5 === 0;
                    accumulatedLength += length;
                    var lineCenter = L.GeometryUtil.destination(segment.getLatLngs()[0], lineAngleFromNorth, accumulatedLength);
                    var lineStart = L.GeometryUtil.destination(lineCenter, lineAngleFromNorth - 90, (major ? 2 : 1) * LT_WIDTH_METERS);
                    var lineEnd = L.GeometryUtil.destination(lineCenter, lineAngleFromNorth + 90, (major ? 2 : 1) * LT_WIDTH_METERS);
                    L.polyline([lineStart, lineEnd], { color: "#000", weight: 1 }).addTo(_this.map);
                    counter++;
                });
            });
            this._allPoints[0].bringToFront();
        };
        __decorate([
            dependency_utils_1.dependsOn("map")
        ], LajiMapWithLineTransect.prototype, "setLineTransect", null);
        __decorate([
            dependency_utils_1.reflect(),
            dependency_utils_1.dependsOn("lineTransect", "translations")
        ], LajiMapWithLineTransect.prototype, "_updateLTLayerContextMenus", null);
        return LajiMapWithLineTransect;
    }(Base));
    return LajiMapWithLineTransect;
} // tslint:disable-line
exports.default = LajiMapWithLineTransect;
